# Python 3.6
#
# this script loops through all dates of the precipitation dataset
# and calculates averages of all values prior to January of the actual
# year and after Jan of the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os

#--------
# START
#--------


# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
nc_path = r'D:\01.Work\00.PhD\00.Database\GlobalAnalysis\Landcover\inNCfile'

# REGRIDDING COORDINATES ==================================================================#
import glob
import xarray as xr

files = glob.glob(os.path.join(nc_path, '*.nc'))

for file in files:
    nc_obj = xr.open_dataset(file)
    # map the longitudes to the correct order
    nc_obj.coords['longitude'] = np.where(nc_obj.coords['longitude'] > 180, nc_obj.coords['longitude'] - 360,nc_obj.coords['longitude'])

    # get the order of the array according to the correct longitudes
    arr1inds = nc_obj.coords['longitude'].argsort()

    # write the outTiffs to the created folder
    out_dir = os.path.join(os.path.split(nc_path)[0], 'inNCfiles_proc')

    varname = 'C3grass'
    # extrat time steps and no data value
    data_type = str(nc_obj.variables[varname].dtype)  # type of dataset (float32)
    no_data = -9900  # nc_obj.variables[varname].missing_value  # get the missed value

    # GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
    # get the epsg code and output file
    epsg_code = 4326
    outfile_ext = ".tif"

    # extract time steps min and max and long
    ts = nc_obj.variables['time'][:]
    lat = nc_obj.variables['latitude'][:]
    lon = nc_obj.variables['longitude'][:]
    xmin, ymin, xmax, ymax = [min(lon.data), min(lat.data), max(lon.data), max(lat.data)]

    # set geotransform
    nx = len(lon)
    ny = len(lat)

    xres = (xmax - xmin) / nx
    yres = (ymax - ymin) / ny

    # geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
    geotransform = (xmin, xres, 0, ymax, 0, -yres)  # if yres, the lat is wrong

    # sort each latitude (lat_id) data according to longitude
    for lat_id in range(len(nc_obj)):
        nc_obj.data[lat_id] = nc_obj[lat_id][arr1inds[:]]

    nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)

#-------------------------------------------------------------------------------
    # enter variable name and extract data and save to dictionary. They can be:
    varname_lst = ['C3grass', 'C4grass', 'C3crop', 'C4crop', 'C3past', 'C4past', 'Urban',
                   'TrpEBF', 'TrpDBF', 'TmpEBF', 'TmpENF', 'TmpDBF', 'BorENF', 'BorDNF',
                   'SecBorDNF', 'SecBorENF', 'SecTmpDBF', 'SecTmpENF', 'SecTmpEBF', 'SecTrpDBF', 'SecTrpEBF',
                   'Savanna', 'Denseshrub', 'Openshrub', 'Tundra', 'Desert', 'PdRI', 'Water']

    varname_lst = ['C3grass']

    for varname in varname_lst:
        print(varname)




        # ================================== CALCULATE THE CHANGES ================================================================#

        # --------------------------------whrite loop for file------------------------------------------#
        # loop over all time steps (start with second) and average data prior to the current ts and equal-after the current ts
        for i in range(1, len(ts) - 1, 1):  # skip the first and last
            # for i in range(1, 7, 1):
            print(first_year + i)

            data_prior = np.mean(
                np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
            data_post = np.mean(
                np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)

            out_prior = os.path.join(out_dir, str(first_year + i) + '_prior' + outfile_ext)
            out_post = os.path.join(out_dir, str(first_year + i) + '_post' + outfile_ext)
            print(out_prior)

            # sort each latitude (lat_id) data according to longitude
            for lat_id in range(len(data_prior.data)):
                data_prior.data[lat_id] = data_prior.data[lat_id][arr1inds[:]]
                data_post.data[lat_id] = data_post.data[lat_id][arr1inds[:]]

            nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
            nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)

        print("Done")

print("Done")







