# Python 3.6
#
# this script loops through all dates of the precipitation dataset
# and calculates averages of all values prior to January of the actual
# year and after Jan of the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import xarray as xr

#--------
# START
#--------


varname_lst = ['cons_dom','cons_irr']

for varname in varname_lst:
    if varname == 'cons_irr':
        varname = 'cons_irr_watergap'
    print(f"Processing: {varname}")

    # CALL THE INPUT DATA ==============================================================================#
    # get the path of dataset
    path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\WaterUse\inNCfiles_gridded_v2'
    file_path = os.path.join(path, str(varname + '.nc'))
    nc_obj = xr.open_dataset(file_path,
                             decode_times=False)  # must open as xarray, not as nc objec   nc_obj = nc.Dataset(file_path,'r', format="NETCDF4")

    no_data = 9.969209968386869e+36  # default

    nx = 720  # ncols
    ny = 360  # nrow
    ts = 480  # 12*40yrs
    # ================================ (1) CONVERT FROM GRIDDED DATA TO 3D ARRAY IN NETCDF ===========================

    lon_keys = np.linspace(-179.75, 179.75, nx).tolist()  # creat the key list of longitude
    lon_vals = range(len(lon_keys))
    lon_dic = dict(zip(lon_keys, lon_vals))

    lat_keys = np.linspace(-89.75, 89.75, ny).tolist()
    lat_vals = range(len(lat_keys))
    lat_dic = dict(zip(lat_keys, lat_vals))

    mons = ts  # len(nc_obj.variables[varname])

    # creat an empty 3D array with lon,lat,time
    # np_array = np.full((len(lon_keys), len(lat_keys), len(mons)), no_data)
    # np_array = np.full((len(mons),len(lat_keys), len(lon_keys)), no_data) #reshape structure time-lat-lon
    np_array = np.full((mons, ny, nx), no_data)  # reshape structure time-lat-lon

    # for mon in range(len(nc_obj.variables[varname])):
    for mon in range(12):
        print("TS:" + str(mon))
        for i in range(len(nc_obj.variables[varname][mon])):
            lat = nc_obj.variables['lat'][i].values.tolist()
            lon = nc_obj.variables['lon'][i].values.tolist()
            val = nc_obj.variables[varname][mon][i].data.tolist()
            # lat_ix = lats.index(lat)  # takes about 40 times longer than using dictionary
            # lon_ix = lons.index(lon)
            try:
                lat_ix = lat_dic[lat]
            except KeyError:
                continue
            try:
                lon_ix = lon_dic[lon]
            except KeyError:
                continue
            # np_array[lon_ix, lat_ix,mon] = val
            np_array[
                mon, lat_ix, lon_ix] = val  # to generate the array with shapes according to dimensions time,lat,lon
    print("Done")

    ### ============================ (2) WRITE 3D array TO NETCDF4 FILE (for later use)  =================================================
    # https://unidata.github.io/netcdf4-python/netCDF4/index.html#netCDF4.Variable
    nc_dir = os.path.join(os.path.split(path)[0], "inNCfiles", str(varname + '.nc'))
    nc_out = nc.Dataset(nc_dir, 'w', 'NETCDF4')  # using netCDF4 for output format

    # creat the dimension
    nc_out.createDimension('lon', nx)
    nc_out.createDimension('lat', ny)
    nc_out.createDimension('time', ts)

    # creat variables
    lonvar = nc_out.createVariable('lon', 'float32', ('lon'));
    lonvar[:] = lon_keys
    latvar = nc_out.createVariable('lat', 'float32', ('lat'));
    latvar[:] = lat_keys
    timevar = nc_out.createVariable('time', 'float64', ('time'));
    timevar[:] = mons

    # creat unit for time
    unout = 'Month since 1971-01-01 00:00:00'
    timevar.setncattr('units', unout)  # date2num(datesout,unout)

    # get the value for the variable
    myvar = nc_out.createVariable(varname, 'float32', ('time', 'lat', 'lon'));
    myvar.setncattr('units', 'mm')
    myvar[:] = np_array
    nc_out.close()  # done
print('Done')

