# Python 3.6
#
# this script loops through all dates of the temperature dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

#import netCDF4 as nc
#from netCDF4 import Dataset
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import glob
import xarray as xr


# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
#file_path = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\EvapoTrans\inNCfiles_aet' #\TerraClimate_aet_1963.nc'
file_path = r'c:\Users\jensk\Work\07_Supervision_Requests\2015_Tinh\181111_GlobalChangePaper\Data\domestic water use\withd_dom.nc'
file_path = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\WaterUse\inNCfiles\domestic water use v2\cons_dom.nc'

#concatenate all files to one
#files = glob.glob(os.path.join(file_path, '*.nc'))
#nc_obj = xr.open_mfdataset(files, combine='by_coords',decode_times=False, chunks={'time': 1}, decode_cf=False)
nc_obj = xr.open_dataset(file_path, decode_times=False)
no_data = -9999
var_name = 'cons_dom'

#nc_obj.variables['lat'][nc_obj.variables['lat'] >= 9e+36] = 0  # set all lats with no data to 0
"""
-There are two variables in each NetCDF file, i.e. lon and lat, which are all in the size of (67420,1):

>>> lat.data

array([-16.25,  65.25,  65.75, ...,  68.75,  70.75,  71.25])     size of (67420,1)

>>> lon.data

array([-179.75, -179.75, -179.75, ...,  179.75,  179.75,  179.75])    size of (67420,1)

So, when you combine lon and lat, you can get the coordinate for global 67420 grids. For example, the coordinate for the first grid cell is (-16.25, -179.75), and for the last grid cell is (71.25, -179.75).
"""

lon_keys = np.linspace(-179.75,179.75,720).tolist()  #creat the key list of longitude
lon_vals = range(len(lon_keys))
lon_dic = dict(zip(lon_keys, lon_vals))

lat_keys = np.linspace(-89.75,89.75,360).tolist()
lat_vals = range(len(lat_keys))
lat_dic = dict(zip(lat_keys, lat_vals))

mons = range(len(nc_obj.variables[var_name]))

np_array = np.full((len(lon_keys), len(lat_keys), len(mons)), no_data)

mon=1
i = 1
#for mon in range(len(nc_obj.variables[var_name])):
for mon in range(12):
    print("TS:"+str(mon))
    for i in range(len(nc_obj.variables[var_name][mon])):
        lat = nc_obj.variables['lat'][i].values.tolist()
        lon = nc_obj.variables['lon'][i].values.tolist()
        val = nc_obj.variables[var_name][mon][i].data.tolist()
        #lat_ix = lats.index(lat)  # takes about 40 times longer than using dictionary
        #lon_ix = lons.index(lon)
        try:
            lat_ix = lat_dic[lat]
        except KeyError:
            continue
        try:
            lon_ix = lon_dic[lon]
        except KeyError:
            continue
        np_array[lon_ix, lat_ix, mon] = val

print("Done")



"""
#GET THE INFOR OF THE DATABASE (1958-2018) ===========================================================================#
first_year = 1958
last_year = 2017
debug = False

varname = 'aet'
out_var = ['outTiffs_',varname]

# create new folder of results to the corresponding varname - CREATE ONCE!
#os.mkdir(os.path.join(os.path.split(file_path)[0],"".join(out_var)))

# write the outTiffs to the created folder
out_dir = os.path.join(os.path.split(file_path)[0],"".join(out_var))

#extrat time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)    #type of dataset (float32)
no_data = -32768  #nc_obj.variables[var_str]._FillValue    #applied for single file


# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin,ymin,xmax,ymax = [min(lon.data),min(lat.data),max(lon.data),max(lat.data)]

#set geotransform
nx = len(lon)
ny = len(lat)

xres = 0.04166666666666666435
yres = 0.04166666666666666435
#(xmax - xmin) / float(nx)   #get the pixel size
#(ymax - ymin) / ny   #get the pixel size

#geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
geotransform = (xmin, xres, 0, ymax, 0, -yres)  #if yres, the lat is wrong


#================================== CALCULATE THE CHANGES ================================================================#
#loop over all time steps (start with the second year) and average data prior to the current ts and equal-after the currents year
last_increment = 0
for i in range(0,len(ts)-12,1): #skip the first and last 12
#for i in range(0,36,1): #skip the first and last 12
    year_increment = int(i/12)
    if debug:
        # get data at 22deg in y and 0deg x from top left
        dx = 0
        dy = 0.04 #int(22/xres)
        print("MON: "+str(i)+"; VAL: "+str(nc_obj.variables[varname][i].data[dx][dy]))

    if not year_increment == last_increment:
        print(first_year+year_increment)
        last_increment = year_increment

    # ERROR IN CALCULATIONS: first year is correct the wrong
    # print out values at a location and check where an error occurs
    data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i],nc_obj.variables[varname][0:i] == no_data), axis=0)
    data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
    if debug:
        #get data at 22deg in y and 0deg x from top left
        dx = 0
        dy = int(22/xres)
        print("YEAR: " + str(year_increment) +
              "; VAL:" + str(data_prior.data[dx][dy]) + "\n")

    #get results : in output directory, namefile.outfileext (here is year_prior.tif)
    out_prior = os.path.join(out_dir,str(first_year + year_increment) + "_prior" + outfile_ext)
    out_post = os.path.join(out_dir, str(first_year + year_increment) + "_post" + outfile_ext)
    print(out_prior)

    #write file to Tiff (int16)
    nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
    nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)
"""
print("Done")

