# Python 3.6
#
# this script loops through all dates of the precipitation dataset
# and calculates averages of all values prior to January of the actual
# year and after Jan of the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import xarray as xr

#--------
# START
#--------

fullvar = 'cons_irr_jpjml'  #watergap'
varname = 'cons_irr'

# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
path = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\WaterUse\inNCfiles_gridded_v2'
file_path = os.path.join(path, str(fullvar+'.nc'))
nc_obj = xr.open_dataset(file_path, decode_times=False)  #must open as xarray, not as nc objec   nc_obj = nc.Dataset(file_path,'r', format="NETCDF4")
nc_obj.variables

no_data = 9.969209968386869e+36   #default

nx = 720 #ncols
ny = 360 #nrow
ts = 480 #12*40yrs
#================================ (1) CONVERT FROM GRIDDED DATA TO 3D ARRAY IN NETCDF ===========================

lon_keys = np.linspace(-179.75,179.75,nx).tolist()  #creat the key list of longitude
lon_vals = range(len(lon_keys))
lon_dic = dict(zip(lon_keys, lon_vals))

lat_keys = np.linspace(-89.75,89.75,ny).tolist()
lat_vals = range(len(lat_keys))
lat_dic = dict(zip(lat_keys, lat_vals))

mons = ts #len(nc_obj.variables[varname])

#creat an empty 3D array with lon,lat,time
#np_array = np.full((len(lon_keys), len(lat_keys), len(mons)), no_data)
#np_array = np.full((len(mons),len(lat_keys), len(lon_keys)), no_data) #reshape structure time-lat-lon
np_array = np.full((mons,ny, nx), no_data) #reshape structure time-lat-lon

#for mon in range(len(nc_obj.variables[varname])):
for mon in range(12):
    print("TS:"+str(mon))
    for i in range(len(nc_obj.variables[varname][mon])):
        lat = nc_obj.variables['lat'][i].values.tolist()
        lon = nc_obj.variables['lon'][i].values.tolist()
        val = nc_obj.variables[varname][mon][i].data.tolist()
        #lat_ix = lats.index(lat)  # takes about 40 times longer than using dictionary
        #lon_ix = lons.index(lon)
        try:
            lat_ix = lat_dic[lat]
        except KeyError:
            continue
        try:
            lon_ix = lon_dic[lon]
        except KeyError:
            continue
        #np_array[lon_ix, lat_ix,mon] = val
        np_array[mon,lat_ix, lon_ix] = val  #to generate the array with shapes according to dimensions time,lat,lon
print("Done")

### ============================ (2) WRITE 3D array TO NETCDF4 FILE (for later use)  =================================================
#https://unidata.github.io/netcdf4-python/netCDF4/index.html#netCDF4.Variable
nc_dir = os.path.join(os.path.split(path)[0],"inNCfiles",str(fullvar+'.nc'))
nc_out = nc.Dataset(nc_dir,'w','NETCDF4') # using netCDF4 for output format

#creat the dimension
nc_out.createDimension('lon',nx)
nc_out.createDimension('lat',ny)
nc_out.createDimension('time',ts)

#creat variables
lonvar = nc_out.createVariable('lon','float32',('lon'));lonvar[:] = lon_keys
latvar = nc_out.createVariable('lat','float32',('lat'));latvar[:] = lat_keys
timevar = nc_out.createVariable('time','float64',('time')); timevar[:]=mons

#creat unit for time
unout = 'Month since 1971-01-01 00:00:00'
timevar.setncattr('units',unout);  #date2num(datesout,unout)

#get the value for the variable
myvar = nc_out.createVariable(varname,'float32',('time','lat','lon')); myvar.setncattr('units','mm')
myvar[:] = np_array
nc_out.close()  #done


### ============================ (3) GET THE NETCDF FILES AND ITS INFOR =================================================
nc_dir = os.path.join(os.path.split(path)[0],"inNCfiles",str(fullvar+'.nc'))
#path = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\WaterUse\inNCfiles'
#file_path = os.path.join(path, str(fullvar+'.nc'))
nc_obj = nc.Dataset(nc_dir, 'r', format="NETCDF4")

#get some value from netcdf file
data_type = str(nc_obj.variables[varname].dtype)  # type of dataset

#GET THE INFOR OF THE DATABASE (1971-2010) --------------------=#
first_year = 1971
last_year = 2010
debug = False

# create new folder of results to the corresponding varname
os.mkdir(os.path.join(os.path.split(path)[0],"outTiffs",fullvar))

# write the outTiffs to the created folder
out_dir = os.path.join(os.path.split(path)[0],"outTiffs",varname)

# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lon = nc_obj.variables['lon'][:]
lat = nc_obj.variables['lat'][:]
xmin, ymin, xmax, ymax = [min(lon), min(lat), max(lon), max(lat)]

#set geotransform
nx = 720 #len(lon)
ny = 360 #len(lat)

xres = (xmax - xmin) / float(nx) #which should get back to original cell size
yres = (ymax - ymin) / float(ny)

#geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
geotransform = (xmin, xres, 0, ymax, 0, -yres)  #if yres, the lat is wrong

#https://www.pythoninformer.com/python-libraries/numpy/index-and-slice/
#====================================== START LOOPING ========================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year
last_increment = 0

for i in range(0,len(ts)-12,1): #skip the first and last 12
#for i in range(0, 12, 1):  # skip the first and last 12
    year_increment = int(i/12)
    if debug:
        # get data at 22deg in y and 0deg x from top left
        dx = 0
        dy = int(22 / xres)
        print("MON: "+str(i)+"; VAL: "+str(nc_obj.variables[varname][i].data[dx][dy]))

    if not year_increment == last_increment:
        print(first_year+year_increment)
        last_increment = year_increment

        # ERROR IN CALCULATIONS: first year is correct then wrong
        # Print out values at a location and check where an error occurs
        data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
        data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)

        if debug:
            # get data at 22deg in y and 0deg x from top left
            dx = 0
            dy = int(22 / xres)
            print("YEAR: " + str(year_increment) + "; VAL: " + str(data_prior.data[dx][dy])+"\n")

        out_prior = os.path.join(out_dir, str(first_year + year_increment) + '_prior' + outfile_ext)
        out_post = os.path.join(out_dir, str(first_year + year_increment) + '_post' + outfile_ext)
        #print(out_prior)

        nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
        nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)
print("Done")


