# Python 3.6
#
# this script loops through all dates of the population dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only
#

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os

#--------
# INPUTS
#--------

file_path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\GPCC\full_data_monthly_v2018_025.nc'
out_dir = os.path.join(os.path.split(file_path)[0], 'outTiffs')
first_year = 1891
debug = False

nc_obj = nc.Dataset(file_path, 'r', format="NETCDF4")
# check file
if not nc_obj.file_format == "NETCDF3_CLASSIC" and not nc_obj.file_format == "NETCDF4_CLASSIC":
    print("Warning: Data Format of netCDF is: " + nc_obj.file_format)
epsg_code = 4326
outfile_ext = ".tif"

# enter variable name and extract data and save to dictionary
varname = 'precip'

# extract time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)
no_data = nc_obj.variables[varname].missing_value

# extract time steps. min and max lat and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin, ymin, xmax, ymax = [min(lon), min(lat), max(lon), max(lat)]

# set geotransform
nx = len(lon)
ny = len(lat)
xres = (xmax - xmin) / float(nx)
yres = (ymax - ymin) / float(ny)
geotransform = (xmin, xres, 0, ymax, 0, -yres)


# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year
last_increment = 0
#for i in range(0,len(ts)-12,1): #skip the first and last 12
for i in range(828,1440, 1):  #only calculate in period 1960-2010 (828=1/1960, 1440=12/2010
    year_increment = int(i/12)
    if debug:
        # get data at 22deg in y and 0deg x from top left
        dx = 0
        dy = int(22 / xres)
        print("MON: "+str(i)+"; VAL: "+str(nc_obj.variables[varname][i].data[dx][dy]))

    if not year_increment == last_increment:
        print(first_year+year_increment)
        last_increment = year_increment

        # ERROR IN CALCULATIONS: first year is correct then wrong
        # Print out values at a location and check where an error occurs
        data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
        data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)

        if debug:
            # get data at 22deg in y and 0deg x from top left
            dx = 0
            dy = int(22 / xres)
            print("YEAR: " + str(year_increment) + "; VAL: " + str(data_prior.data[dx][dy])+"\n")

        out_prior = os.path.join(out_dir, str(first_year + year_increment) + '_prior' + outfile_ext)
        out_post = os.path.join(out_dir, str(first_year + year_increment) + '_post' + outfile_ext)

        nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
        nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)
print("Done")

