# Python 3.6
#
# this script loops through all dates of the population dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only
#

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os

#--------
# INPUTS
#--------

file_path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\GPCC\full_data_monthly_v2018_025.nc'
out_dir = os.path.join(os.path.split(file_path)[0], 'outTiffs')
first_year = 1891
debug = False

nc_obj = nc.Dataset(file_path, 'r', format="NETCDF4")
# check file
if not nc_obj.file_format == "NETCDF3_CLASSIC" and not nc_obj.file_format == "NETCDF4_CLASSIC":
    print("Warning: Data Format of netCDF is: " + nc_obj.file_format)
epsg_code = 4326
outfile_ext = ".tif"

# enter variable name and extract data and save to dictionary
varname = 'precip'

# extract time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)
no_data = nc_obj.variables[varname].missing_value

# extract time steps. min and max lat and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin, ymin, xmax, ymax = [min(lon), min(lat), max(lon), max(lat)]

# set geotransform
nx = len(lon)
ny = len(lat)
xres = (xmax - xmin) / float(nx)
yres = (ymax - ymin) / float(ny)
geotransform = (xmin, xres, 0, ymax, 0, -yres)




#tham khao tu nguon khac----------------
x = np.random.random([100,100,100])
y = np.random.random([100,100,100])
c_sort = np.argsort(x, axis = 2)
#f = y[c_sort]
idx = np.argsort(x,axis=2)
m,n = y.shape[:2]
f = y[np.arange(m)[:,None,None], np.arange(n)[:,None], idx]




#https://stackoverflow.com/questions/55072697/memoryerror-vs-valueerror-array-is-too-big-when-allocating-a-large-ndarray-on
#========================== BAT DAU TINH TOAN =====================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year
last_increment = 0

#ADD NEW INDEX TO AVOID MEMMORY ERROR
#idx = nc_obj.variables['lon'][:].argsort()

start_year = 1960
end_year= 2010

#for i in range(0,len(ts)-12,1): #skip the first and last 12
#for i in range(828,1440, 1):  #only calculate in period 1960-2010 (828=1/1960, 1440=12/2010
for year in range(start_year, end_year):
    print(f"Processing year: {year}")

    # -------------------------------------------------------- calculation from Jens ---------------------------------------#
    # Print out values at a location and check where an error occurs
    ds_pre = ds_in.sel(time=slice(f"{start_year}-01-01", f"{year}-12-31"))
    ds_post = ds_in.sel(time=slice(f"{year}-01-01", f"{end_year}-12-31"))

    pre_mean_arr = np.mean(ds_pre, 0)
    post_mean_arr = np.mean(ds_post, 0)

    # data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
    # data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)


    out_prior = os.path.join(out_dir, str(year) + '_prior' + outfile_ext)
    out_post = os.path.join(out_dir, str(year) + '_post' + outfile_ext)
    # --------------------------------------------------------------------------------------------------------------------------#

    nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
    nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)
print("Done")

os.path.join(out_dir, str(year) + '_prior' + outfile_ext)
os.path.join(out_dir, str(year) + '_post' + outfile_ext)
