# Python 3.6
#
# this script loops through all dates of the temperature dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only


import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os

# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
file_path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\Temperature\air.mon.mean.v501.nc' #on PC

#Creat the direction to save output data (save same directory, but creat new folder named "outTiffs'
out_dir = os.path.join(os.path.split(file_path)[0],'outTiffs')


# REGRIDDING COORDINATES (if needed) ==================================================================#
import xarray as xr
nc_obj = xr.open_dataset(file_path)

#map the longitudes to the correct order
nc_obj.coords['lon'] = np.where(nc_obj.coords['lon']>179.75,nc_obj.coords['lon']-360,nc_obj.coords['lon'])

# get the order of the array according to the correct longitudes
arr1inds = nc_obj.coords['lon'].argsort()

#GET THE INFOR OF THE DATABASE (1900-2017) ===========================================================================#
first_year = 1900
last_year = 2017
debug = False

#enter variable name and extract data and save to dictionary
varname = 'air'

#extrat time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)   #type of dataset (float32)
no_data = -9.96921e+36    #nc_obj.variables[varname]._FillValue  #get the missed value


# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin,ymin,xmax,ymax = [min(lon.data),min(lat.data),max(lon.data),max(lat.data)]

#set geotransform
nx = len(lon)
ny = len(lat)

xres = (xmax-xmin)/float(nx)
yres = (ymax-ymin)/float(ny)

geotransform = (xmin,xres,0,ymax,0,-yres)

for i in range(720,1320, 1):
    y = int(i/12)
    print(first_year+y)
#================================== CALCULATE THE CHANGES ================================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year
last_increment = 0
#for i in range(0,len(ts)-12,1): #skip the first and last 12
for i in range(720,1320, 1):  # skip the first and last 12
    year_increment = int(i/12)
    if debug:
        # get data at 22deg in y and 0deg x from top left
        dx = 0
        dy = int(22 / xres)
        print("MON: "+str(i)+"; VAL: "+str(nc_obj.variables[varname][i].data[dx][dy]))

    if not year_increment == last_increment:
        print(first_year+year_increment)
        last_increment = year_increment

        # ERROR IN CALCULATIONS: first year is correct then wrong
        # Print out values at a location and check where an error occurs
        data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
        data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)

        if debug:
            # get data at 22deg in y and 0deg x from top left
            dx = 0
            dy = int(22 / xres)
            print("YEAR: " + str(year_increment) + "; VAL: " + str(data_prior.data[dx][dy])+"\n")

        out_prior = os.path.join(out_dir, str(first_year + year_increment) + '_prior' + outfile_ext)
        out_post = os.path.join(out_dir, str(first_year + year_increment) + '_post' + outfile_ext)
        print(out_prior)

        # sort data according to longitude
        for id in range(len(data_prior.data)):
            data_prior.data[id] = data_prior.data[id][arr1inds[:]]
            data_post.data[id] = data_post.data[id][arr1inds[:]]

        nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
        nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)
print("Done")

