# Python 3.6
#
# this script loops through all dates of the precipitation dataset
# and calculates averages of all values prior to January of the actual
# year and after Jan of the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only
#

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os

#--------
# INPUTS
#--------

file_path = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\Population\population_histsoc_5min_annual_1861-2005.nc'
#file_path = r'c:\CommonData\world\#GlobalAnalysis\Population\population_histsoc_5min_annual_1861-2005.nc'
out_dir = os.path.join(os.path.split(file_path)[0], 'outTiffs_5min')
first_year = 1861
last_year = 2005

nc_obj = nc.Dataset(file_path, 'r', format="NETCDF4")
# check file
if not nc_obj.file_format == "NETCDF3_CLASSIC" and not nc_obj.file_format == "NETCDF4_CLASSIC":
    print("Warning: Data Format of netCDF is: " + nc_obj.file_format)
epsg_code = 4326
outfile_ext = ".tif"

# enter variable name and extract data and save to dictionary
varname = 'population'

# extract time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)
no_data = nc_obj.variables[varname].missing_value
data_type
# extract time steps. min and max lat and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin, ymin, xmax, ymax = [min(lon), min(lat), max(lon), max(lat)]
xmin,ymin,xmax,ymax

# set geotransform
nx = len(lon)
ny = len(lat)
xres = (xmax - xmin) / float(nx)
yres = (ymax - ymin) / float(ny)
geotransform = (xmin, xres, 0, ymax, 0, -yres)
#geotransform = (-180, 0.5, 0, 90, 0, -0.5)

# loop over all time steps (start with second) and average data prior to the current ts and equal-after the current ts
for i in range(1,len(ts)-1,1): #skip the first and last
    print(first_year+i)

    data_prior = np.mean(np.ma.masked_array(nc_obj.variables[varname][0:i], nc_obj.variables[varname][0:i] == no_data), axis=0)
    data_post = np.mean(np.ma.masked_array(nc_obj.variables[varname][i:-1], nc_obj.variables[varname][i:-1] == no_data), axis=0)

    out_prior = os.path.join(out_dir, str(first_year + i) + '_prior' + outfile_ext)
    out_post = os.path.join(out_dir, str(first_year + i) + '_post' + outfile_ext)

    #np_array = np.zeros((720, 360), dtype=np.uint8)

    nhf.WriteNumpyToTiff(data_prior.data, geotransform, ny, nx, epsg_code, data_type, out_prior)
    nhf.WriteNumpyToTiff(data_post.data, geotransform, ny, nx, epsg_code, data_type, out_post)

print("Done")


