# Python 3.6
#
# this script loops through all dates of the temperature dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only


import netCDF4 as nc
from netCDF4 import Dataset
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import glob
import xarray as xr


# CALL THE INPUT DATA ==============================================================================#

#get the path of dataset
nc_dir = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\EvapoTrans\inNCfiles_aet' #\TerraClimate_aet_1963.nc'
#out_dir = r'C:\0. VT.Tinh\1. PhD in CAU Kiel\2. Database_important\GlobalAnalysis\EvapoTrans\mtlyTiffs_aet'

# filter input files
all_files = os.listdir(nc_dir)
nc_files = nhf.filter_nc_files(all_files)

# get id lat and long from first nc file
nc_obj = Dataset(os.path.join(nc_dir, nc_files[0]))


#GET THE INFOR OF THE DATABASE (1958-2018) ===========================================================================#
varname = 'aet'
out_var = ['mtlyTiffs_',varname]

# create new folder of results to the corresponding varname - CREATE ONCE!
os.mkdir(os.path.join(os.path.split(nc_dir)[0],"".join(out_var)))

# write the outTiffs to the created folder
out_dir = os.path.join(os.path.split(nc_dir)[0],"".join(out_var))

#extrat time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)    #type of dataset (float32)
no_data = -32768  #nc_obj.variables[var_str]._FillValue    #applied for single file


# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin,ymin,xmax,ymax = [min(lon.data),min(lat.data),max(lon.data),max(lat.data)]

#set geotransform
nx = len(lon)
ny = len(lat)

xres = 0.04166666666666666435
yres = 0.04166666666666666435
#(xmax - xmin) / float(nx)   #get the pixel size
#(ymax - ymin) / ny   #get the pixel size

#geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
geotransform = (xmin, xres, 0, ymax, 0, -yres)  #if yres, the lat is wrong


#================================== calculation ================================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year

monthly_averages = True #to create long-term monthly averages
total_averages = True #to create one total average over all months and years


# loop over 12 months
if monthly_averages:
    for i in range(12):
        mon = i+1
        print("Month: "+str(mon))

        #get data from net cdf
        first_nc_obj = Dataset(os.path.join(nc_dir, nc_files[0]))
        av_array = np.array(nc_obj.variables[varname][i])
        np.ma.masked_values(av_array, no_data)

        # loop over all nc_files
        for nc_file in nc_files[1:]:
            # open nc file
            nc_obj = Dataset(os.path.join(nc_dir, nc_file))
            tot_array = np.array(nc_obj.variables[varname])
            if len(tot_array) != 12:
                print(str(nc_file)+' is not 12 months long!!!')
                sys.exit()
            mon_array = np.ma.masked_values(tot_array[i], no_data)
            av_array = np.add(mon_array,av_array)
        av_array = np.divide(av_array, 12)

        # save to tiff
        outraster_str = os.path.join(out_dir, "av_"+varname+"_"+str("%02d" % (i+1))+".tif")
        nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)


if total_averages:
    is_first = True
    for nc_file in nc_files:
        # open nc file
        print(nc_file)
        nc_obj = Dataset(os.path.join(nc_dir, nc_file))
        tot_array = np.array(nc_obj.variables[varname])

        start_mon = 0 #start at first mnonth
        if len(tot_array) != 12:
            print(str(nc_file) + ' is not 12 months long!')
            sys.exit()
        if is_first:
            av_array = np.array(nc_obj.variables[varname][0])  # array of first netCDF and first months
            np.ma.masked_values(av_array, no_data)

            outraster_str = os.path.join(out_dir, "FirstFileFirstMon_" + varname + ".tif")
            nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

            start_mon = 1 # for the first file, start at month 2 because month 1 is already in av_array
            is_first = False
        for i in range(start_mon,12):
            print(i)
            print(process.memory_info().rss)
            mon_array = np.ma.masked_values(tot_array[i], no_data)
            av_array = np.add(mon_array, av_array)
    av_array = np.divide(av_array, len(nc_files)*12)

    # save to tiff
    outraster_str = os.path.join(out_dir, "av_" + varname + ".tif")
    nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)
