# Python 3.6
#
# this script loops through all dates of the temperature dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only


import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os


# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
file_path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\GPCC\full_data_monthly_v2018_025.nc'

#Creat the direction to save output data (save same directory, but creat new folder named "outTiffs'
out_dir = os.path.join(os.path.split(file_path)[0],'mtlyTiffs')


# REGRIDDING COORDINATES (if needed) ==================================================================#
import xarray as xr
nc_obj = xr.open_dataset(file_path)


#GET THE INFOR OF THE DATABASE (1900-2017) ===========================================================================#
debug = False

#enter variable name and extract data and save to dictionary
varname = 'precip'

#extrat time steps and no data value
data_type = str(nc_obj.variables[varname].dtype)   #type of dataset (float32)
no_data = -9.96921e+36    #nc_obj.variables[varname]._FillValue  #get the missed value


# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lat = nc_obj.variables['lat'][:]
lon = nc_obj.variables['lon'][:]
xmin,ymin,xmax,ymax = [min(lon.data),min(lat.data),max(lon.data),max(lat.data)]

#set geotransform
nx = len(lon)
ny = len(lat)

xres = (xmax-xmin)/float(nx)
yres = (ymax-ymin)/float(ny)

geotransform = (xmin,xres,0,ymax,0,-yres)

#================================== calcu ================================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year

monthly_averages = True #to create long-term monthly averages
total_averages = True #to create one total average over all months and years


# loop over 12 months
if monthly_averages:
    for i in range(12):
        mon = i+1
        print("Month: "+str(mon))

        #get data from net cdf
        av_array0 = np.array(nc_obj.variables[varname][i])
        av_array = np.ma.masked_array(av_array0, no_data)

        outraster_str = os.path.join(out_dir, "av_"+varname+"_"+str("%02d" % (i+1))+".tif")
        nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

if total_averages:
    is_first = True
    av_array = np.divide(av_array, 12)

    # save to tiff
    outraster_str = os.path.join(out_dir, "av_total_" + varname + ".tif")
    nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)
print("Done")