# Python 3.6
#
# this script loops through all dates of the temperature dataset
# and calculates averages of all values prior to the actual year
# and after the actual year including the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

import netCDF4 as nc
from netCDF4 import Dataset
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import glob
import xarray as xr


# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
file_path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\EvapoTrans\inNCfiles_pet' #\TerraClimate_aet_1963.nc'

#GET THE INFOR OF THE DATABASE (1958-2018) ===========================================================================#
first_year = 1959
last_year = 2010
debug = False

varname = 'pet'
out_var = ['TiffsTS_',varname]

# create new folder of results to the corresponding varname - CREATE ONCE!
os.mkdir(os.path.join(os.path.split(file_path)[0],"".join(out_var)))

# write the outTiffs to the created folder
out_dir = os.path.join(os.path.split(file_path)[0],"".join(out_var))


#================================== calcu ================================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year

monthly_averages = False #to create long-term monthly averages
total_averages = False #to create one total average over all months and years
annual_averages = True #to create annual time series

# loop over 12 months
if monthly_averages:
    for i in range(12):
        mon = i+1
        print("Month: "+str(mon))

        #get data from net cdf
        av_array0 = np.array(nc_obj.variables[varname][i])
        av_array = np.ma.masked_array(av_array0, no_data)

        outraster_str = os.path.join(out_dir, "av_"+varname+"_"+str("%02d" % (i+1))+".tif")
        nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

if total_averages:
    is_first = True
    av_array = np.divide(av_array, 12)

    # save to tiff
    outraster_str = os.path.join(out_dir, "av_total_" + varname + ".tif")
    nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

 # loop over all time steps (start with second) and average data prior to the current ts and equal-after the current ts
if annual_averages:
    for year in range(first_year, last_year+1):
        print(f"Processing year: {year}")

        file = 'TerraClimate_pet_'+str(year)+'.nc'
        nc_obj = xr.open_dataset(os.path.join(file_path,file))

        # extrat time steps and no data value
        data_type = str(nc_obj.variables[varname].dtype)  # type of dataset (float32)
        no_data = -32768  # nc_obj.variables[var_str]._FillValue    #applied for single file

        # GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
        # get the epsg code and output file
        epsg_code = 4326
        outfile_ext = ".tif"

        # extract time steps min and max and long
        ts = nc_obj.variables['time'][:]
        lat = nc_obj.variables['lat'][:]
        lon = nc_obj.variables['lon'][:]
        xmin, ymin, xmax, ymax = [min(lon.data), min(lat.data), max(lon.data), max(lat.data)]

        # set geotransform
        nx = len(lon)
        ny = len(lat)

        xres = 0.04166666666666666435
        yres = 0.04166666666666666435
        # (xmax - xmin) / float(nx)   #get the pixel size
        # (ymax - ymin) / ny   #get the pixel size

        # geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
        geotransform = (xmin, xres, 0, ymax, 0, -yres)  # if yres, the lat is wrong
        mean_dat = np.mean(np.ma.masked_array(nc_obj.variables[varname], nc_obj.variables[varname]== no_data), axis=0)
        outfile = os.path.join(out_dir, str(year) + ".tif")
        nhf.WriteNumpyToTiff(mean_dat.data, geotransform, ny, nx, epsg_code, data_type, outfile)
    print("Done")
print("Done")





