# Python 3.6
#
# this script loops through all dates of the precipitation dataset
# and calculates averages of all values prior to January of the actual
# year and after Jan of the actual year
#
# the resulting tiffs are named with year - where "prior" does
# not include the year and "post" includes the year
#
# all values are means only

import netCDF4 as nc
import NetCDF_HelperFunctions as nhf
import numpy as np
import os
import xarray as xr

#--------
# START
#--------

varname = 'cons_irr_pcr'
varname = 'cons_dom'

# CALL THE INPUT DATA ==============================================================================#
#get the path of dataset
path = r'D:\01. Work\00. PhD\00. Database\GlobalAnalysis\WaterUse\inNCfiles'

no_data = 9.969209968386869e+36   #default

file_path = os.path.join(path, str(varname+'.nc'))
nc_obj = nc.Dataset(file_path, 'r', format="NETCDF4")

#get some value from netcdf file
data_type = str(nc_obj.variables[varname].dtype)  # type of dataset

#GET THE INFOR OF THE DATABASE (1971-2010) --------------------=#
first_year = 1971
last_year = 2010
debug = False

# write the outTiffs to the created folder
out_dir = os.path.join(os.path.split(path)[0],"TiffsTS_"+varname)

# create new folder of results to the corresponding varname
os.mkdir(out_dir)

# GEOTRANSFORM THE INPUT/OUTPUT DATA ======================================================================#
#get the epsg code and output file
epsg_code = 4326
outfile_ext = ".tif"

#extract time steps min and max and long
ts = nc_obj.variables['time'][:]
lon = nc_obj.variables['lon'][:]
lat = nc_obj.variables['lat'][:]
xmin, ymin, xmax, ymax = [min(lon), min(lat), max(lon), max(lat)]

#set geotransform
nx = 720 #len(lon)
ny = 360 #len(lat)

xres = (xmax - xmin) / float(nx) #which should get back to original cell size
yres = (ymax - ymin) / float(ny)

#geotransform = (top_left_x, w-e pixel resolution, 0, top_left_y, 0, n-s pixel resolution (negative value))
geotransform = (xmin, xres, 0, ymax, 0, -yres)  #if yres, the lat is wrong



#================================== calcu ================================================================#
# loop over all time steps (start with second year) and average data prior to the current year and equal-after the current year

monthly_averages = False #to create long-term monthly averages
total_averages = False #to create one total average over all months and years
annual_averages = True #to create annual time series

first_year = 1959 #1770
last_year = 2010

# loop over 12 months
if monthly_averages:
    for i in range(12):
        mon = i+1
        print("Month: "+str(mon))

        #get data from net cdf
        av_array0 = np.array(nc_obj.variables[varname][i])
        av_array = np.ma.masked_array(av_array0, no_data)

        outraster_str = os.path.join(out_dir, "av_"+varname+"_"+str("%02d" % (i+1))+".tif")
        nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

if total_averages:
    is_first = True
    av_array = np.divide(av_array, 12)

    # save to tiff
    outraster_str = os.path.join(out_dir, "av_total_" + varname + ".tif")
    nhf.WriteNumpyToTiff(av_array.data, geotransform, ny, nx, epsg_code, data_type, outraster_str)

 # loop over all time steps (start with second) and average data prior to the current ts and equal-after the current ts
if annual_averages:
    for year in range(first_year, last_year+1):
        print(f"Processing year: {year}")
        dat = nc_obj.sel(time=slice(f"{year}-01-01", f"{year}-12-31"))
        mean_dat = np.mean(dat, 0)

        outfile = os.path.join(out_dir, str(year) + ".tif")
        nhf.WriteNumpyToTiff(mean_dat[varname].to_masked_array(), geotransform, ny, nx, epsg_code, data_type, outfile)
    print("Done")
print("Done")




