
#Author: Tinh Vu (tvu@hydrology.uni-kiel.de)


# Tinh Vu
# Write results to the 2 folders:
#              Res0_Precook
#              Res1_Changepoint
#              Res1_Trend
#              Res2_outTiffsRres

scriptName <- "GA01_Main_Calculation.R"
source("C:/Users/tinhv/Dropbox/PhD/Paper3_GlobalAnalysis/P3_Analysis/_RCoding/_supportFuncs/_callData.R")

dataSet <- "B"
damSet <- "B"
flow <- "MEAN"
out_tiff <- paste0(out_dir,'/Res2_outTiffsRres'); out_tiff

if(!file.exists(out_tiff)) dir.create(out_tiff)
readme <- paste0( "\n","Input are: \n", 
                  "Changpoint set: ", dataSet,"\n",
                  "Flow type: ", flow,"\n", "\n",
                  "Output save to: ", out_tiff, "\n", "\n",
                  "Extracted from R Script: \n", 
                  paste0(path, "/",scriptName),"\n")
writeLines(readme,
           paste0(out_tiff,"/_READme.txt"))




#====================================  ***STEP 1*** - Changepoint detection  ==================================== 
source(paste0(path,"/_supportFuncs/Func01_CPT.R"))
out_dir_cpt <- paste0(out_dir,'/Res1_Changepoint')


#------ Step 1a - calculate single changepoint - done -------------
gsim_lst <- as.character(gsimFull$gsim.no)#[1:50]

flow =  "MEAN"
dataSet = "B"

# for (flow in flow_lst){
  res <- detectCPT(gsim_lst, 
                   Qpath,
                   dataSet,
                   flow)
  write.table(res,file = paste0(out_dir_cpt,'/CPT',dataSet, "_", flow,'.Rres'), row.names = FALSE, sep = ",")
# }


#------ Step 1b - combine the final changepoint - done -------------
source(paste0(path,"/_supportFuncs/Func01_CPT.R"))
files <- list.files(out_dir_cpt,
                    pattern = ".Rres", full.names = TRUE)
files <- files[!grepl("Final_", files)]

for(f in files){
  cat(paste0('processing ', basename(f), '\n'))
  res <- finalCPT(f)
  
  res_join <- left_join(gsimFull[colnames(gsimFull) %in% c('gsim.no','longitude','latitude', 'continent')],
                   res,
                   by="gsim.no")
  write.table(res,
              file = paste0(out_dir_cpt,'/Final_', basename(f)), row.names = FALSE, sep = ",")
}


#====================================  STEP 1.2 - Trend detection - done  ==================================== 

flow =  "MEAN" 
dataSet_lst = c("A", "B", "C")
for (dataSet in dataSet_lst){
  res <- detectTrend(gsim_lst,
                     Qpath,
                     dataSet,
                     flow)
  write.table(res,file = paste0(out_dir,'/Res1_Trend/Trend',dataSet, "_", flow,'.Rres'), row.names = FALSE, sep = ",")
}



#====================================  ***STEP 2*** - Extract variables ==================================== 
#this includes extracting:
#a) whole variables +  RandomVar.Rres
#b) Discharge
#c) GRanD
#d) calculate Ratio from b) and c)

#----------- 2.1 Extract Pre/Post Variables-------------------------------
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R"))

gsim_cpt <- read.table(paste0(cptPath,"/Final_CPT_set", dataSet,"_",flow,".Rres"), header = TRUE, sep = ",")

VARlst = c('Temperature', 'Precipitation')#,'EvapoTrans', 'WaterUse') #landcover will be calc in 2.1b
VAR = VARlst[2]

for(VAR in VARlst){
  ivar_lst <- switch(VAR,
                     Precipitation = "GPCC",
                     Temperature = "Temperature",
                     EvapoTrans = "pet",
                     #----------------------------------------------------------------------------------#
                     #old list, based on original class --> extract pre/post  --> grouping them to smaller groups
                     # Landcover = c('C3grass', 'C4grass','C3crop', 'C4crop','C3past', 'C4past','Urban',
                                   # 'TrpEBF','TrpDBF','TmpEBF','TmpENF','TmpDBF','BorENF','BorDNF',
                                   # 'SecBorENF','SecBorDNF','SecTrpEBF','SecTrpDBF','SecTmpEBF','SecTmpENF','SecTmpDBF',
                                   # 'Savanna','Denseshrub','Openshrub','Tundra','Desert','PdRI','Water'),
                     #----------------------------------------------------------------------------------#
                     WaterUse = c('cons_irr_watergap','cons_dom') #'cons_irr_pcr','cons_irr_jpjml','cons_irr_h08',
  ); ivar_lst; ivar <- ivar_lst[1]
  
  for (ivar in ivar_lst) {
    res <- extVar(gsim_cpt,
                  VAR, #c('Temperature', 'Precipitation','EvapoTrans','Landcover','WaterUse')
                  GlobalDATA,
                  RDATADIR)
    
    ivar_new <- ivar
    if(grepl("cons_irr_watergap", ivar)) ivar_new <- "Irrigation"
    if(grepl("cons_dom", ivar)) ivar_new <- "WaterUse"
      
    # print(head(res))
    out_file <- ifelse(VAR %in% c('Precipitation','Temperature','EvapoTrans'),
                       paste0(out_tiff,'/',VAR,'.Rres'),
                       paste0(out_tiff, '/',VAR,'_',ivar,'.Rres')); out_file
    print(out_file)
    write.table(res,
                file = out_file, row.names = F, sep = ",")
    }#end each var
}#end loop for VAR


#----------- *2.1b Extract Pre/Post NEW landcover (5 class 09.04.2023)-------------------------------
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R")) #----- DONE ----#

precookLC_dir <- paste0(GlobalDATA, "/Landcover/outTiffs_reclassify")
LC_dirs <- list.dirs(paste0(GlobalDATA, "/Landcover/inNCfiles_reclassify"),
                    recursive = F,
                    full.names = T)
LC_lst <- basename(LC_dirs)
gsim_cpt <- read.table(paste0(cptPath,"/Final_CPT_set", dataSet,"_",flow,".Rres"), header = TRUE, sep = ",")

bg <- Sys.time()
for(LC in LC_lst){
  res <- extrLCnew(gsim_lst,
                   gsim_cpt,
                   LC,
                   precookLC_dir)

  #write output to file
  out_file <- paste0(out_tiff, "/", LC, ".Rres")
  write.table(res,
              file = out_file, row.names = FALSE, sep = ",")
}; print(Sys.time()-bg)




#----------- 2.2 Extract Pre/Post RandomVar.Rres -------------------------------
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R"))

#get the change point detection
gsim_cpt <- read.table(paste0(cptPath,"/Final_CPT_set", dataSet,"_",flow,".Rres"), header = TRUE, sep = ",")

# dataSet_lst = c("B") #A, C can not be run due to limite values in 1960-2010
res <- extRandVar(gsim_cpt,
                  GlobalDATA,
                  RDATADIR)

out_file <- paste0(out_tiff,'/RandomVar.Rres'); out_file
write.table(res, 
            file = out_file, row.names = FALSE, sep = ",") 



#----------- 2.3 Extract Pre/Post Discharge.Rres-------------------------------
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R"))


res_dis <- extrDis(gsim_cpt,
                   Qpath)

out_file <- paste0(out_tiff,'/Discharge.Rres'); out_file
write.table(res_dis, 
            file = out_file, row.names = FALSE, sep = ",")  



#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
#                 2.4 Extract Dam capacity pre/post CPT each basin      ----
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R"))
# added on 09.04.2023
# this is to extract capcity based on pre/post of changes
# the dataset (A, B, C) doent match with damSet, thus, just only damset B was selected to calchanges
  
gsim_lst <- gsimFull$gsim.no
grand <- read.table(paste0(out_dir,"/Res0_Precook/GRanD_list_full.csv"), header = T, sep = ";")

dataSet <- "B"
flow = "MEAN"
gsim_cpt <- read.table(paste0(cptPath,"/Final_CPT_set", dataSet,"_",flow,".Rres"), header = TRUE, sep = ",")


option_lst <- c("Full", "NW", "W"); opt <- option_lst[1]; i = gsim_lst[1]
damSet <- "B"

# for(damSet in c("A","B",  "C", "D")){
for (opt in option_lst){
  df <- extDamCAP(gsim_lst,
                  damSet,
                  grand,
                  opt)

  out_file <- paste0(out_tiff, "/Dam", damSet,"_", opt, ".Rres"); out_file
  if(grepl("_Full.", out_file)) out_file <- gsub("_Full", "", out_file)
  write.table(df,
              file = out_file, row.names = FALSE, sep = ",")
}#close for option
# }#close for damsat
# 



#====================================  STEP 3 - Ratio and water volume ==================================== 
#this includes extracting:  ---> Datafiles will be copied to outTiffRres_clean folder
#) RATIO OF DAM (NoDam, WithDam)
#) wATER VOLUME
source(paste0(path,"/_supportFuncs/Func02_ExtrVariables.R"))

dis <- fread(paste0(out_tiff,'/Discharge.Rres'))
d_files <- list.files(out_tiff, ".Rres", full.names = T)
d_files <- d_files[grepl("Dam", d_files)]; d_files
d_files <- d_files[!grepl("tio", d_files)]; d_files #to remove Caclulate RatioDam

f <- d_files[1]; f

for(f in d_files){
  df <- fread(f)
  dfr <- right_join(dis,df[, -2],'gsim.no')
  
  #calculate the damType
  # cap (10^6 m3) - theo huong dan cua GranD, page 8, Technical Doku
  # W = Q.T (m3)
  dfr$pre_ratio <- ifelse(is.na(dfr$pre_avg)==TRUE | dfr$pre_avg ==0,
                          NA,
                          (dfr$cap_pre*1000000/(dfr$pre_avg*365*24*60*60))*100) # x100 --> to name it as percentage
  dfr$post_ratio <- ifelse(is.na(dfr$post_avg)==TRUE | dfr$post_avg ==0,
                           NA,
                           (dfr$cap_post*1000000/(dfr$post_avg*365*24*60*60))*100)
  
  dfr <- dfr[,c('gsim.no',"cpt_ID",'pre_ratio','post_ratio')]
  
  #saving results to disk
  out_file <- paste0(out_tiff, "/Ratio", basename(f)); out_file
  
  write.table(dfr,
              file = out_file, row.names = FALSE, sep = ";")
}


#--------------------  WATERVOLUME -------------------------------------#
source(paste0(path,"/_supportFuncs/Func03_CalcWaterVolume_Ratio.R"))

area <-  gsimFull[, c("gsim.no", "area")]

volume_res <- calcVolume(area,
                         flow,
                         dataSet,
                         out_tiff)
out_file <- paste0(out_tiff, "/WaterVolume.Rres"); out_file
write.table(volume_res, out_file, row.names = FALSE, sep = ",")




#------------------xxxxxxxxxxx NO MORE NEED xxxxxxxxxxxxxxxxxxxxx ------------------------------------------



#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
#                 2.5 Calculate mean D.Rres      ----1
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
#calculate mean discharge to plot water volume to global map in paper (NOT USE IN ANALYSIS)
# dataSet_lst = c("A", "B", "C")
flow <- "MEAN"

for(dataSet in dataSet_lst){
  res_meanQ <- extMeanDis(dataSet,
                          flow,
                          gsim_lst,
                          Qpath)
  
  out_file <- paste0(out_dir,'/Res3_MainAnalysis/MeanQ', dataSet,"_",flow,
                     '.Rres')
  write.table(res_meanQ,
              file = out_file, row.names = FALSE, sep = ",")
}




#====================================  BACK UP SCRIPT, NO MORE NEED   ===========================================
# 
# 
# #-----  STEP 4 - Combine Landcover Variable - to smaller groups  ---------------#
# source(paste0(path,"/_supportFuncs/Func04_CleanVariable.R"))
# period <- c('pre_avg','post_avg')
# 
# # write NatureForest/SecondaryForest/Shrubland.Rres 
# VARlist <- c('NatureForest',"SecondaryForest","Shrubland")
# cbineForest(VARlist,
#             period,
#             out_tiff,
#             out_tiff_clean)        
# 
# 
# # write Grassland/Cropland/Pastland.Rres in folder outTiffsRres_clean
# VARlist <- c('grass', 'crop','past')
# Clist <- c('C3', 'C4')
# # ----> write to out_res <- paste0(out_tiff_clean,'_',flow,'/',cp,'.Rres') with cp = 'Grassland', 'Cropland','Pastureland'
# cbineC3C4(VARlist,
#           Clist,
#           flow,
#           period,
#           out_tiff,
#           out_tiff_clean)
# 
# # extract no vegetation --> # write Novegetation.Rres in folder outTiffsRres_clean
# lc_listed <- c('Grassland','Cropland','Pastureland','Shrubland','Urbanland','NatureForest','SecondaryForest')
# lc_rest <- c('Savanna','Tundra','Desert','PdRI','Water') 
# period_lst <- c('Pre','Post')
# 
# #1-summary of all the value from original files  --> there may have many basin has >90% of desert ---> not explainable
# #2-they are calculatef by minus 100 for the current one --> IN USE
# noVeg(flow,
#       option=2, #to define which method that want to extract value
#       period,
#       lc_listed,
#       lc_rest,
#       out_tiff,
#       out_tiff_clean)
# 
# #------ group landcover to 4 groups -----------#
# #add on 22.01.2021 
# land.list <- c("Forest","Urbanisation",'Agriculture','NoVegetation')
# cbineLANDnew(land.list,
#              out_tiff_clean)
# 
# checkLAND(land.list,
#           out_tiff_clean,
#           stat = "max")




















