
#Author: Tinh Vu (tvu@hydrology.uni-kiel.de)


#===========================================================================================#
###                  >>> [Q3] - MOST EXPLANABLE FACTORS  <<< ---- 
#                     Finding most explainable factors 
#===========================================================================================#
#https://rstudio-pubs-static.s3.amazonaws.com/240657_5157ff98e8204c358b2118fa69162e18.html
# https://www.data-to-viz.com/caveat/spider.html

if(TRUE) {
  rm(list = ls())
  source("C:/Users/tinhv/Dropbox/PhD/Paper3_GlobalAnalysis/P3_Analysis/_RCoding/_supportFuncs/_callData.R")
  source(paste0(path,"/_supportFuncs/Func05b_RandomForest.R"))
  source(paste0(path,"/_supportFuncs/Func06_ThemePlots.R"))
  source(paste0(path,"/_supportFuncs/_callVarList.R"))

  
  #list dataset used for this analysis ---  only for info ----#
  dataSet <- "B"
  damSet <- "B"
  flow <- "MEAN"

  change_lst <- c("perc") # "pre", "post", "diff", "absdiff", 
  typChange <- "perc" 
  # cpt_opt <- "strong" #can be NULL, "strong"  "pending" "weak"  # ----- must be consistent for both 3.1 and 3.2 !!!
  static <- c("Altitude", "Area")
  VAR_full <- c(climate, landcover, wateruse, dam, static, "Discharge" ,"RandomVar")
}



###=========================================================================#
###--- 3.0 - Initial set to run/plot  -----
###=========================================================================#
#set V0: original = full - static
#set V1: full = original + static
#set V2: full - (water use + dam)
#set V3: full - (water use + dam + static)


#------START ------------
trainOpt <- "Train" # "noTrain"

#option to rerun RF or not
rerunRF <- F
replotRF <- T
plotRFtoWorldMap <- F
calcRadarRF <- T
plotRadarRF <- T

#0. select option to run random Forest
clus_option = "climatezone" #can ben continents
subCluster = "strong"  #can be strong/pending/weak/all
optCPT = F  #default is NULL, only applicable for continent_all to do cluster by strength of cpt


if(TRUE){
  #create new folder (if needed) --> based on option to calculate Randomforest
  if(isTRUE(rerunRF)){
    out_dir_RF <- paste0(out_fig, "/Q3_CART/_by_climatezones_", subCluster, "_", gsub("-", "",Sys.Date()))
  } else {
    out_dir_RF <- paste0(out_fig, "/Q3_CART/_by_climatezones_strong_20230617")
  }; out_dir_RF
  
  if(grepl("continent", clus_option)) out_dir_RF <- gsub("climatezones", clus_option, out_dir_RF)
  if(isTRUE(optCPT) & subCluster == "all") out_dir_RF <- paste0(out_dir_RF, "_byCPT"); out_dir_RF
  
  if(!file.exists(out_dir_RF)) dir.create(out_dir_RF)
  
  setV_lst <- c("setV1") #, "setV0",  "setV2", "setV3")
  setVAR <- "setV1"

  out_file_RF <- paste0(out_dir_RF, '/RF_to', water,'_',typChange, "_", setVAR); out_file_RF

  VAR_rf <- selSet(setVAR, dataFull = VAR_full) #Func5b
  VAR_plot <- VAR_rf[!grepl("Discharge", VAR_rf)]; VAR_plot; VAR_rf
}

###=========================================================================#
###--- 3.1 (re-)Run RandomForest and save to table  -----
###=========================================================================#
if(isTRUE(rerunRF)){
  Rres <- callRres(option = typChange,    #Func5b
                   out_dir,
                   rename_list = c('area', 'altitude'))
  
  #2. clusters --------------------------------------------------
  if(grepl('continent', clus_option)){
    Rres <- getCluster_byCont(data = Rres,
                              option = subCluster,
                              optCPT)
  } else { #by climate
    Rres <- getCluster(data = Rres, #Func5b
                       subCluster) #can be strong/pending/weak/all
  }
  
  #get number of each cluster (only to visualize it)--------------
  x <- t(as.data.frame(table(Rres$clusters)))
  write.table(x, paste0(out_dir_RF, "/NumberofClusters.csv"), row.names = F, col.names = F, sep = ";")
  
  
  # 3. Random forest ----------------------------------------------
  res_RF <- calcRF(data = Rres,
                  targetVar = water, #Func5b
                   option = clus_option,
                   m = 8, #da test tu 10-13; pattern khong thay doi nhieu! --  should be 5-25
                   trainOpt = trainOpt,#NULL=train, "n
                   ptrain = 0.67) 
  
  # 4. save RF to table ----------------------------------
  file_out <- paste0(out_file_RF, '.csv'); file_out
  if(trainOpt == "noTrain") file_out <- gsub(".csv", paste0("_", trainOpt, ".csv"), file_out); file_out
  
  rfres <- saveRFres(res_RF, option = "RF") #func5b
  write.table(rfres, file_out, row.names = FALSE,sep = ',')
  
  kgeres <- saveRFres(res_RF, option = "KGE") #Func5b
  write.table(kgeres,
              file = gsub("RF_to", "KGE_RF_to", file_out),
              row.names = FALSE,sep = ',')
  
  nseres <- saveRFres(res_RF, option = "NSE") #Func5b
  write.table(nseres,
              file = gsub("RF_to", "NSE_RF_to", file_out),
              row.names = FALSE,sep = ',')
  
} #end of rerunRF 
  
###=========================================================================#
###--- [3.2] - Fig 8: Plot importance to Heatmap (from table in 3.1) -----
###=========================================================================#
if(isTRUE(replotRF)){

  file_in <- paste0(out_dir_RF, '/RF_toDischarge_perc_setV1.csv')
  fig_out <- paste0(out_dir_RF, "/", gsub("-", "", Sys.Date()), 
                    "_based_on_",
                    gsub(".csv", "", basename(file_in)),'.png'); fig_out
  if(trainOpt == "noTrain") fig_out <- gsub(".png", paste0("_", trainOpt, ".png"), fig_out); fig_out
  
  res <- read.table(file_in, header=TRUE,sep = ',')
  df_res <- reshape2::melt(res,'VAR')
  
  #find the differences with RandVar to visualize to plot ---------------#
  fil <- res
  nr <- nrow(fil)
  for(i in 1:nr){
    for (j in 2:ncol(fil)){
      fil[i,j] <- fil[i,j] - fil[nr,j]
    }
  }
  df_fill <- reshape2::melt(fil,'VAR')
  names(df_fill)[3] <- "fill"
  

  #join two table and prepare data to plot --------------------------------------#
  df_plot <- dplyr::left_join(df_res, df_fill)
  
  # df_plot$factor <- as.character(df_plot$variable)
  #rename the factors and order it 
  for(i in 1:nrow(df_plot)) {
    df_plot$group[i] <- ifelse(df_plot$VAR[i] %in% climate,'Climatic factors',
                               ifelse(df_plot$VAR[i] %in% landcover,'Landcover',
                                      ifelse(df_plot$VAR[i] %in% c(wateruse, dam),'Water infrastructure',
                                             ifelse(df_plot$VAR[i] %in% static,'Static factors',
                                                    ""))))
    
    if(clus_option == "continent"){
      df_plot$factor[i] <- gsub("AF|AS|AU|EU|NA|SA", "", df_plot$variable[i])
    } else {
      df_plot$factor[i] <- substr(df_plot$variable[i], 1,1)
    }
  }
  
  if(!isTRUE(optCPT) & clus_option == "continent") {
    df_plot$factor <- ifelse(grepl("D1", df_plot$factor),
                             "Pristine condition (D1)",
                             ifelse(grepl("D2", df_plot$factor),
                                    "Stable regulated (D2)",
                                    "Unstable regulated (D3)"))
  } else {
    df_plot$factor <- ifelse(grepl("S", df_plot$factor),
                             "Strong",
                             ifelse(grepl("P", df_plot$factor),
                                    "Pending",
                                    "Weak"))
  }

  #added 17.06.2023 ------add text below randomvar -----------------#
  res_txt <- res

  #--***display text below RandomVar as () or - ----
  for(j in 2:ncol(res_txt)){
    rv <- res_txt[13,j]
    for(i in 1:12){
      val <- as.numeric(res_txt[i,j])
      if(val < rv) val <- paste0("(", val, ")") #"-"
      res_txt[i,j] <- val
    }
  }
  
  df_txt <- reshape2::melt(res_txt,'VAR'); names(df_txt)[3] <- "text" 
  df_plot <- full_join(df_plot, df_txt)
  
  
  #fix order ----------------------------------#
  df_plot$VAR <- factor(df_plot$VAR,levels = VAR_plot, ordered = T)
  df_plot$group <- factor(df_plot$group,levels = c('Climatic factors', 'Landcover', 'Water infrastructure','Static factors',""),ordered = T)
  
  if(!isTRUE(optCPT) & clus_option == "continent") {
    df_plot$factor <- factor(df_plot$factor, levels = c("Pristine condition (D1)",
                                                        "Stable regulated (D2)", 
                                                        "Unstable regulated (D3)"), ordered = T)
  } else {
    df_plot$factor <- factor(df_plot$factor, levels = c("Strong", "Pending", "Weak"), ordered = T)
  }
  
  
  #x-axis for continent plot
  if(clus_option == "continent"){
    df_plot$xaxis <- substr(df_plot$variable, 1, 2)
    df_plot$xaxis <- ifelse(grepl("AF", df_plot$xaxis), "Africa",
                            ifelse(grepl("AS", df_plot$xaxis), "Asia",
                                   ifelse(grepl("AU", df_plot$xaxis), "Australia",
                                          ifelse(grepl("EU", df_plot$xaxis), "Europe",
                                                 ifelse(grepl("NA", df_plot$xaxis), "North America",
                                                        "South America")))))
  }
  if(grepl("climate", clus_option)){
    df_plot$variable <- as.character(lapply(as.character(df_plot$variable), renameClus)) #Func 5b
    df_plot$variable <- getLevelsClusters(df_plot$variable) #func5b
  }
  
# a=df_plot
  
  #---------------------- *Fig.9 [HEATMAP] - Select variable to plot-----------------------#
  library(plyr); library(dplyr)
  ifelse(typChange == 'perc', color <- 'dodgerblue4', color <- 'darkred')
  
  
  library(ggplot2); library(stringr)
  
  p <- ggplot(df_plot, aes(variable, VAR))
  if(clus_option == "continent") p <- ggplot(df_plot, aes(xaxis, VAR))
  
  p <- p +
    geom_tile(aes(fill = fill))  +
    geom_text(aes(label= text),color='black',size= 5) + #size = 4.5 
    scale_fill_gradient2(low = "darkorange2", mid = "white", high ="cyan4",na.value = "grey85" ,
                         guide = "colourbar", aesthetics = "fill",
                         limits = c(round(min(na.omit(df_plot$fill)),digits = -1),
                                    round(max(na.omit(df_plot$fill)),digits = -1)+10),
                         breaks=seq(round(min(na.omit(df_plot$fill)),digits = -1),
                                    round(max(na.omit(df_plot$fill)),digits = -1)+10,
                                    10)) +
    
    scale_x_discrete(labels = function(x) str_wrap(x, width = 10)) +
    scale_y_discrete(limits=rev) + 
    ylab("") +
    
    facet_grid(group ~ factor, scales="free", space="free")+
    
    guides(fill = guide_colourbar(label.theme = element_text(colour = color, angle = 0,size = 15),
                                  title = 'Lower/Higher than RandomVar (%): ',direction = "horizontal", title.position = "left",
                                  title.hjust = 0.5,
                                  label.hjust = 0.5, label.vjust = 1)) +
    
    theme_bw()+  theme(legend.position= "bottom",
                       legend.text = element_text(size = 15),
                       legend.key.width = unit(1.5,'cm'),
                       legend.title = element_text(size = 15,face = 'bold'),
                       plot.title = element_text(size = 15,face = 'bold'),
                       axis.title.x =element_blank(),
                       axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                       axis.text.y = element_text(size = 15),
                       axis.text.x = element_text(size = 15,angle = 45,hjust = 1, vjust = 1),
                       panel.grid.major.x =  element_line(colour='grey'),
                       panel.grid.minor = element_blank(),
                       strip.background = element_blank(),strip.text = element_text(size = 15,face = "bold")); p
  
  
  w = 1200
  h = 850 #750
  if(clus_option != "continent" & subCluster == "all"){
    w = 1500 #2100
    h = 800
  }
  png(fig_out, width = w, height = h, type = "windows")
  print(p)
  dev.off()
}#end of auto-relot  
  
  

###=========================================================================#
###--- [3.3] - Fig 10 - Plot Radarchart per continent -----
###=========================================================================#

rmbelowRV <- T #remove all values below RV

#3.3a. Get Max-Min-Average all clusters by basins -----------------
if(isTRUE(calcRadarRF)){
  # f <- paste0(out_fig, "/Q3_CART/_by_climatezones_strong/20230527_RF_toDischarge_perc_setV1.csv")
  f <- paste0(out_dir_RF, "/RF_toDischarge_perc_setV1.csv")
  res <- as.data.frame(fread(f))
  
  #important of variables below RV --> 0
  if(isTRUE(rmbelowRV)){
    for(j in 2:ncol(res)){
      rv <- res[13,j]
      for(i in 1:12){
        val <- as.numeric(res[i,j])
        if(val < rv) val <- 0
        res[i,j] <- val
      }
    }
  }
  
  
  #remove RandomVar
  res <- res[-13, ]
  tres <- as.data.frame(t(res))
  names(tres) <- tres[1,]
  tres$clusters <- rownames(tres)
  tres <- tres[-1, ]
  
  #comnbine with continent- -----------------------------#
  clus <- read.table("Clusters_climate_Strong.Rres", header=TRUE,sep = ',')
  clus <- dplyr::left_join(clus, gsimFull[, c("gsim.no", "continent")])
  clus<- dplyr::left_join(clus, tres, 'clusters')
  
  z <- lapply(cont_lst, function(x){
    df <- clus[clus$continent == x, 
               4:15]
    df <- as.data.frame(sapply(df, as.numeric))
    
    xx <- data.frame(min = sapply(df, min),
                     max = sapply(df, max),
                     avg = sapply(df, mean))
    
    k <- as.data.frame(t(xx))
    k$continent <- x
    k
    
  })
  res <- do.call(rbind, z)
  res <- res[grepl("avg", rownames(res)),]
  rownames(res) <- res$continent
  
  #save to disk --> Fig 11 (excel) -----------------------------------------
  
  fname <- gsub(".csv", "_grouped_to_continent.csv", basename(f))
  if(isTRUE(rmbelowRV)) fname <- gsub(".csv", "_rmbelowRV.csv", fname)
  write.table(res,
              paste0(out_dir_RF, "/", fname), sep = ",", row.names = F)
  
  #-- 2. find max min ---------------#
  clus_sub <- clus[, 4:15]
  clus_sub <- as.data.frame(sapply(clus_sub, as.numeric))
  maxmin <- data.frame(max = sapply(clus_sub, max),
                       min = sapply(clus_sub, min),
                       avg = sapply(clus_sub, mean)
  )
  maxmin$VAR = rownames(maxmin)
  
  fname <- gsub(".csv", "_grouped_to_continent_maxmin.csv", basename(f))
  if(isTRUE(rmbelowRV)) fname <- gsub(".csv", "_rmbelowRV.csv", fname)
  write.table(maxmin,
              paste0(out_dir_RF, "/", fname), sep = ",", row.names = F)
  
}#end autorun in radardchart


#3.3b. Plot Radarchart ---------------------------------------
rmbelowRV <- F #remove all values below RV

if(isTRUE(plotRadarRF)){
  # f <- paste0(out_fig, "/Q3_CART/_by_climatezones_strong/20230527_RF_toDischarge_perc_setV1_grouped_to_continent.csv")
  f <- paste0(out_dir_RF, "/RF_toDischarge_perc_setV1_grouped_to_continent.csv")
  if(isTRUE(rmbelowRV)) f <- gsub(".csv", "_rmbelowRV.csv", f)
  
  res <-  as.data.frame(fread(f))
  rownames(res) <- res$continent
  # Variables summary ------------------------------#
  res <- res[, -13]
  # Get the minimum and the max of every column  
  col_max <- apply(res, 2, max)
  col_min <- apply(res, 2, min)
  # Calculate the average profile 
  col_mean <- apply(res, 2, mean)
  # Put together the summary of columns
  col_summary <- t(data.frame(Max = col_max, Min = col_min, Average = col_mean))
  
  
  # Bind variables summary to the data
  res2 <- as.data.frame(rbind(col_summary, res))
  head(res2)
  
  colors <- c("#00AFBB", "#E7B800", "#FC4E07", 
                       "darkorchid4", "dodgerblue3", "green")
                       # "chartreuse4", "darkcyan", "darkgoldenrod4")
  # "red","blue","green")
  
  # Plot in radarchart ---------------------------#
  date <- strsplit(basename(f), "_")[[1]][1]
  figname <- paste0(out_dir_RF, "/", gsub("-","",Sys.Date()), '_Radarchart_based_on_res_in',date,'.png'); figname
  if(isTRUE(rmbelowRV)) figname <- gsub(".png", "_rmbelowRV.png", figname)
  
  png(figname,
      width = 1200, height = 900, type = "windows") #3col,x2row = w1800, h950
  
  if(TRUE){
    opar <- par() 
    # Define settings for plotting in a 3x4 grid, with appropriate margins:
    par(mar = rep(0.5,4)) #4 =4 canh cua figure
    par(mfrow = c(2,3))
    library(fmsb)
    # Produce a radar-chart for each student
    for (i in 4:nrow(res2)) {
      radarchart(
        res2[c(1:3, i),
             c(1, 12:2)],
        # pfcol = c("#99999980", NA), 
        pfcol = scales::alpha(c("#99999980", colors[i-3]), 0.35),
        pcol= c(NA, "black"), #colour for outline in polygon
        plty = 2, #line type
        plwd = 2, #width of line
        vlcex = 2, #font size for variable name
        axistype = 1, #display text on axis
        # caxislabels=seq(0,30,5),
        calcex = 2,
        # color = colors[i-3],
        # title = row.names(res2)[i]
      )
      mtext(side = 3, #1=below, 3=above, 2=left, 4=right
            line = -2.5, #distance with figure
            at = -0.75, #position of title -1=left side, 0=middle, 1= right side
            cex = 1.5, #fontsize of title
            row.names(res2)[i],
            font = 4 #1=normal, 2=bold, 3=italic, 4=bold+italic
      )
    }
    # Restore the standard par() settings
    par <- par(opar) 
    dev.off()
  }#end autoplot radarchart

  
}


  
  

  ###=========================================================================#
  ###--- [3.3b] - Plot Wordcloud per continent -----
  ###=========================================================================#
  res <- as.data.frame(fread(paste0(out_dir_RF, "/20230527_RF_toDischarge_perc_setV1_grouped_to_continent.csv")))
  
  c <- cont_lst[2]
  
  png('TEST.png', width = 900, height = 600, type = "windows") #3col,x2row = w1800, h950
  par(mfrow=c(2,3)) #row, col

  library(wordcloud); library(ggwordcloud); library(wordcloud2)
  for (c in cont_lst) {
    print(c)
    df <- res[, c("VAR", c)]
    names(df)[2] <- "cont"
    
    wordcloud(words = df$VAR,
              freq = df$cont,
              # colors = colorlist,
              min.freq = 1,
              max.words=200, random.order = F, rot.per=0.25,
              colors = brewer.pal(8, "Dark2"))

    mtext(c, line=-2, cex=1.5, adj = 0.1)
  }
 dev.off()
  
 
 
 
  ###=========================================================================#
  ###--- [3.3] - Plot importance to world map rom table in 3.1) -----
  ###=========================================================================#
  if(isTRUE(plotRFtoWorldMap)){
    
    file_in <- paste0(out_file_RF, '.csv'); file_in
    if(trainOpt == "noTrain") file_in <- gsub(".csv", paste0("_", trainOpt, ".csv"), file_in); file_in
    
    
    fig_out <- paste0(out_file_RF, '_worldmap.png'); fig_out
    if(trainOpt == "noTrain") fig_out <- gsub(".png", paste0("_", trainOpt, ".png"), fig_out); fig_out
    
    res <- read.table(file_in, header=TRUE,sep = ',')
    
    #remove RandomVar
    res <- res[-13, ]
    
    #------------- a. Finding dominant variable ----------
    #find max
    mx <- as.data.frame(apply(res, 2, max))
    mx$clusters <- rownames(mx); mx=mx[-1,]; names(mx)[1] <- "VarImpt"
    mx$VarImpt <- as.numeric(mx$VarImpt)
    
    for(i in 1:nrow(mx)){
      cl <- mx$clusters[i]
      sig <- res[, c("VAR", cl)]; names(sig)[2] <- "clus"
      v <- sig$VAR[sig$clus == mx$VarImpt[i]]
      mx$domVAR[i] <- v
    }
    res_domVAR <- mx
    #------------- b. Finding dominant groups --- not yet done ----------
    res$groups <- ifelse(grepl(paste(climate, collapse = "|"), res$VAR), "Climate", 
                         ifelse(grepl(paste(landcover, collapse = "|"), res$VAR), "Landcover",
                                ifelse(grepl(paste(c(wateruse, dam), collapse = "|"), res$VAR), "Water Infrastructure",
                                       "Static")))
    grp_lst <- unique(res$groups); gr = grp_lst[1]
    
    z <- lapply(grp_lst, function(gr){
      r <- res[res$groups == gr, ]; r=r[, -c(1, 19)]
      r <- as.data.frame(apply(r, 2, sum)); names(r) <- gr
      r$clusters <- rownames(r)
      r
    })
    res_grp <- do.call(cbind, z)
    res_grp <- res_grp[ , c("clusters", "Climate", "Landcover", "Water Infrastructure", "Static")]
    
    # for(i in 1:nrow(res_grp)){
    #   cl <- res_grp$clusters[i]
    #   mx <- res[i, 2:5]
    #   sig <- res[, c("VAR", cl)]; names(sig)[2] <- "clus"
    #   v <- sig$VAR[sig$clus == mx$VarImpt[i]]
    #   mx$domVAR[i] <- v
    # }
    # res_domVAR <- mx
    
    #------------ plot dominant var to world map --------------
    Rres <- callRres(option = typChange,    #Func5b
                     out_dir,
                     rename_list = c('area', 'altitude'))
    if(grepl('continent', clus_option)){
      Rres <- getCluster_byCont(data = Rres,
                                option = subCluster)
    } else { #by climate
      Rres <- getCluster(data = Rres, #Func5b
                         subCluster) #can be strong/pending/weak/all
    }
    
    gsim_dat <- Rres[, c("longitude", "latitude", "clusters")]
    for(i in 1:nrow(gsim_dat)){
      gsim_dat$domGrp[i] <- res_domVAR$domVAR[res_domVAR$clusters ==gsim_dat$clusters[i]]
    }
    
    
    p <- baseworld2() +  
      geom_point(data = gsim_dat,
                 aes(x = longitude, 
                     y = latitude,
                     size = factor(domGrp),
                     colour = factor(domGrp), #, levels = c("strong", "pending", "weak"),
                     # labels = c("Strong", "Pending", "Weak")
                 )
      ) +
      themeCPT_global(pts_size = c(4, 6),
                      color_lst = c("goldenrod2", "yellowgreen","palegreen1", "cyan4", "pink"),
                      title = "Dominant variable:"); p
    
    
    png(fig_out, width = 1800, height = 1800, type = "windows")
    print(p)
    dev.off()
    
    
    
  }
  
# }#end autorun --------


# ###=========================================================================#
# ###--- [3.3] - Plot KGE -----
# ###=========================================================================#
#   #call RF values from step 3.1
# setVAR <- "setV1"
cr_lst <- c("KGE", "NSE"); cr = cr_lst[1]

for(cr in cr_lst){
  
  fig_name <-  paste0(out_dir_RF,'/_groups_by_climatezones/RF_Performance_to',water,'_',typChange,"_", cr,'_climatezones.png') ; fig_name
  if(trainOpt == "noTrain") fig_name <- gsub(".png", paste0("_", trainOpt, ".png"), fig_name); fig_name
  
  
l <- lapply(setV_lst, function(x){
# if(TRUE){
  fres <- paste0(out_dir,'/Res3_Main_Analysis/3.1_RF_VarImpt/Area_',cr,'_RF_to', water,'_',typChange, "_", x, '_climatezones.csv'); fres
  if(trainOpt == "noTrain") fres <- gsub(".csv", paste0("_", trainOpt, ".csv"), fres); fres

  
  res <- read.table(fres,header=TRUE,sep = ',')
  res[, 1] <- round(res[, 1], 1)
  res$setV <- x
  res
})
res <- as.data.frame(rbindlist(l)); names(res)[1] <- "criteria"
if(grepl("KGE", cr)){
  res$fill <- ifelse(res$criteria < -0.41, "bad",
                     ifelse(res$criteria > 0.41, "good", "acceptable")) 
} else {
  res$fill <- ifelse(res$criteria < 0.3, "bad",
                     ifelse(res$criteria > 0.6, "good", "acceptable"))
}

res$Clusters<- factor(res$Clusters,levels = c("SD1A","SD1B", "SD1Cf", "SD1Cs", "SD1Cw", "SD1Df", "SD1Ds", "SD1Dw", "SD1E",
                                              "SD2",  "SD3", 
                                              "PD1",  "PD2", "PD3",  "WD1", "WD2",  "WD3" ),
                      ordered = T)

  library(ggplot2); library(stringr)
  p <- ggplot(res, aes(Clusters, setV)) +  #Median, sim
    geom_tile(aes(fill = fill))  +
    geom_text(aes(label= criteria),color='black',size=4.5)+
    
    scale_fill_manual(values = c("orange", "red", "cyan4", "grey")) +

    ylab("") + 
    theme_bw() +  theme(legend.position= "bottom",
                        legend.text = element_text(size = 15),
                        legend.key.width = unit(1.5,'cm'),
                        legend.title = element_text(size = 15,face = 'bold'),
                        plot.title = element_text(size = 15,face = 'bold'),
                        axis.title.x =element_blank(),
                        axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                        axis.text.y = element_text(size = 15),
                        axis.text.x = element_text(size = 15,angle = 0,hjust = 0.5),
                        panel.grid.major.x =  element_line(colour='grey'),
                        panel.grid.minor = element_blank(),
                        strip.background = element_blank(),strip.text = element_text(size = 15,face = "bold")); p
  
  
  png(filename = fig_name,
      width = 1200, height = 750, type = "windows")
  print(p)
  dev.off()
  
}

#================== END ======================================






##====================================================#
###--- Example of Regression Trees ----
###====================================================#

cat('     a) ploting example... /n')

water <- 'Discharge'
Rres_lst <- c(climate,wateruse,landcover,dam, water)
Rres <- callRres(option = typChange,    #Func5b
                 out_dir,
                 rename_list = c('area', 'altitude'))

# Rres <- read.table(paste0(out_dir,'/Res3_Main_Analysis/VariableChanges_',flow,'.Rres'),header = TRUE,sep = ',')
df <- Rres[is.na(Rres$cpt_ID)==FALSE,
           colnames(Rres) %in% c(Rres_lst,  "Area", "RandomVar", 'continent')]; head(df)



cont <- 'North America'
dfb <- df[df$continent == cont, 
          -which(names(df) %in% "continent")] #remove the factor column

head(dfb)
c=substr(cont,1,2)

library(rpart); library(rpart.plot)

png(paste0(out_fig,'/Q3_CART/Fig5_RTexamples_',c,'.png'),width = 1300, height = 400, type = "windows") #Af=500,900
if(TRUE) {
  par(mfrow=c(1,2))
  #-------------------------------------------------------------~
  #set 1: missing Temperature and ratio dam
  set1 <- c(climate,landcover, dam,"RandomVar",water) #------> missing wateruse, static
  # set1 <- c(climate,  wateruse, "Area", "RandomVar",water)
  dfb1 <- dfb[colnames(dfb) %in% set1 ]; head(dfb1)
  m1 <- rpart(Discharge~., data = dfb1, method = 'anova')
  rpart.plot(m1,type = 2,digits = 3,fallen.leaves = TRUE,box.palette="-GnYlRd")#box.palette=c("paleturquoise", "salmon1"))
  
  #set 2: wateruse missing
  set2 <- c(landcover, "Area", "RandomVar",water)
  dfb2 <- dfb[colnames(dfb) %in% set2]; head(dfb2)
  m2 <- rpart(Discharge~., data = dfb2, method = 'anova') #GnYlRd;
 
  rpart.plot(m2,type = 2,digits = 3,fallen.leaves = TRUE,box.palette="-GnYlRd")#box.palette=c("paleturquoise", "salmon1"))
  
  dev.off()
}




#------------------------ SET 1 -------------------------------~

#extract the importance of variable
x <- as.data.frame(m1$variable.importance)
dt1 <- data.frame(var = Rres_lst)

if(nrow(x)==0) {#some variable could not find the importance value, thus may occur an error
  dt1 <- data.frame(dt,var_importance =0)
} else {
  x$var <- row.names(x)
  x$var_importance <- round(x$`m1$variable.importance`*100/sum(x$`m1$variable.importance`),1)
  
  dt1 <- dplyr::full_join(dt1,x,'var')
  dt1[is.na(dt1)==TRUE] <- 0
}#close for non-importance in some variable
dt1$miss <- 'wtuse+statistic'
colnames(dt1)[1:2] <- c('VAR','varimpt')

rownames(dt1) <- dt1$VAR

### Keep the row in order 
library(tidyr)
rows <- unique(c(climate,landcover,wateruse,dam,  "Altitude", "Area", "RandomVar"))

dt1 <- dt1[rows,]


#----------------------- SET 2 ---------------------------------~
#extract the importance of variable
x <- as.data.frame(m2$variable.importance)
dt2 <- data.frame(var = Rres_lst)

if(nrow(x)==0) {#some variable could not find the importance value, thus may occur an error
  dt2 <- data.frame(dt,var_importance =0)
} else {
  x$var <- row.names(x)
  x$var_importance <- round(x$`m2$variable.importance`*100/sum(x$`m2$variable.importance`),1)
  
  dt2 <- dplyr::full_join(dt2,x,'var')
  dt2[is.na(dt2)==TRUE] <- 0
}#close for non-importance in some variable

dt2$miss <- 'wateruse'
colnames(dt2)[1:2] <- c('VAR','varimpt')


rownames(dt2) <- dt2$VAR

### Keep the row in order 
library(tidyr)

dt2 <- dt2[rows,]

dt <- rbind(dt1,dt2)

write.table(dt, paste0("C:/Users/tinhv/Dropbox/PhD/Paper3_GlobalAnalysis/P3_Writing/Fig.5_RTexamples_VarImpt_NA.csv"),
            row.names = FALSE,sep = ',')





