
#Author: Tinh Vu (tvu@hydrology.uni-kiel.de)

#+++++++++++++++++++ THIS IS TO PREPARE WHOLE GSIM INFOR FOR CALCULATION +++++++++++++++++++++++++++

source("C:/Users/tinhv/Dropbox/PhD/Paper3_GlobalAnalysis/P3_Analysis/_RCoding/_supportFuncs/_callPath.R")
source("C:/Users/tinhv/Dropbox/PhD/Paper3_GlobalAnalysis/P3_Analysis/_Rcoding/_supportFuncs/Func00_General.R")

### load the catchmetn characteristic
char <- read.table(paste0(GlobalDATA,'/GSIM_metadata/GSIM_catchment_characteristics.csv'), header = TRUE, sep = ",") ##write results to tile

###=================================================#
###--- 1) NA check -----
###=================================================#
#call full gsim
gsimOrig <- read.csv(paste0(GlobalDATA,'/GSIM_metadata/GSIM_metadata.csv'),header = TRUE,sep = ",")
#call the all files data
all_files <- list.files(Qpath, #paste(file_path,"/GSIM_metadata/GSIM_indices/TIMESERIES/yearly",sep = "/")
                        pattern = '.year',full.names = TRUE)

#xem function in support files
lres <- lapply(all_files, na.check)       #take about 6 mins
lres_df <- rbindlist(lres)
gsim_join <- dplyr::full_join(gsimOrig,lres_df,by = "gsim.no")

#write to file
write.csv(gsim_join,
          paste(out_dir,"Res0_Precook/GSIM1_NA_check.csv",sep = "/"),
          col.names = TRUE,row.names = FALSE,sep = ",")
rm(gsim_join)



###============================================================#
###===== 2) Modify area to make sure whole basin having areas ----
###============================================================#
# frist, extract based on estimated area from characteristics data
# in case of missing areas, --> calc from shapefile (but most of these are from basin, 24 basins having no area)

gsimCheck <- read.table(paste(out_dir,"Res0_Precook/GSIM1_NA_check.csv",sep = "/"),
                         header = TRUE, sep = ",")
area_list <- c(1,100,100000) #folow Liebcher 1993
z <- list()
# !!! IMPORTANT !!! please keep the gs files as from (2)
for (i in 1:nrow(gsimCheck)) {
  #get the original area if provided
  area <- gsimCheck$area[i]
  gsim.no <- gsimCheck$gsim.no[i]
  print(gsim.no)
  
  if (!is.na(area)) area <- char[match(i, char$gsim.no),'area.est']
  
  #if no area is found in character  ->>call it from shapefiles 
  if(is.na(area)){
    noRdata_files <- readLines(paste0(path,"/noRdata.lst"))
    
    if (gsim.no %in% noRdata_files) {  
      print('NA - Rdata files not available')
      area = NA
      clas = NA
      
    }else{
      load(paste0(RDATADIR, "/",  gsub(".shp", "",gsim.no), ".RData"))
      area <- raster::area(tmp_shp)/1000000 #this is completely similar as by raster::area
    }
  }
  
  clas <- ifelse(area<= area_list[1], "Micro",
                 ifelse(area > area_list[1] & area <= area_list[2],'Small',
                        ifelse(area > area_list[2] & area <= area_list[3], 'Large','Macro')))

  gsimCheck$area[i] <- area
  gsimCheck$area_clas[i] <- clas
}#close for all

# res <- gsimCheck[names(gsimCheck) %in% c('gsim.no','area','area_clas')]
write.table(gsimCheck,file = paste0(out_dir,'/Res0_Precook/GSIM2_add_area.csv'), row.names = FALSE, sep = ",") ##write results to tile
rm(gsimCheck)


###============================================================#
###===== 3) Add continent base on lon/lat ----
###============================================================#
#note: long.new-lat.new has 12174 NA values for all the continent --> consider the 
gsimCheck <- read.table(paste(out_dir,"Res0_Precook/GSIM2_add_area.csv",sep = "/"),
                        header = TRUE, sep = ",")

lonlat <- gsimCheck[,c('longitude','latitude')]

library(rworldmap)
# The single argument to this function, points, is a data.frame(long,lat)
coords2continent = function(points) {  
  countriesSP <- getMap(resolution='low')
  pointsSP = SpatialPoints(points, proj4string=CRS(proj4string(countriesSP)))  
  indices = over(pointsSP, countriesSP)
  indices$REGION   # returns the continent (7 continent model)
}
gsimCheck$continent <-  coords2continent(lonlat)

#!!!important!!! must convert to character otherwise the Europe is always missing
gsimCheck$continent <- as.character(gsimCheck$continent)

##add some other continent
for (i in 1:nrow(gsimCheck)) {
  if(is.na(gsimCheck$continent[i])==TRUE) {
    ctr <- as.character(gsimCheck$country[i])
    
    ifelse(ctr %in% c('CA','US'), con <- 'North America',
           ifelse(ctr %in% c('GF','BR','PF'),con <- 'South America',
                  ifelse(ctr == 'ZW',con <- 'Africa',
                         ifelse(ctr=='ID', con <- 'Asia',
                                ifelse(ctr=='AU', con <- 'Australia',con <- 'Europe')))))
    gsimCheck$continent[i] <- con
  } else {
    gsimCheck$continent[i] <- gsimCheck$continent[i] 
  }#close for NA continent
}

write.table(gsimCheck, paste0(out_dir,'/Res0_Precook/GSIM3_add_continent.csv'),row.names = FALSE,sep = ',')
rm(gsimCheck)


###============================================================#
###===== 4) Write clean gsim  ----
###============================================================#
gsimClean <- read.table(paste(out_dir,"Res0_Precook/GSIM3_add_continent.csv",sep = "/"), header = TRUE, sep = ",") #39059 basins

#no Rdatafiles
noShps <- readLines(paste0(path, "/noRdata.lst"))

#remove all stations having no Shp
gsimClean1 <- gsimClean[!grepl(paste(noShps, collapse = "|"), 
                               gsimClean$gsim.no), ]
write.table(gsimClean1,paste0(out_dir,'/Res0_Precook/GSIM4_clean_full.csv'),row.names = FALSE,sep = ',')
rm(gsimClean)



# 
# ###============================================================#
# ###===== 5) Narrow the selection for gsim ----
# ###============================================================#
# gsimClean <- read.table(paste(out_dir,"Res0_Precook/GSIM4_clean_full.csv",sep = "/"), header = TRUE, sep = ",") #39059 basins
# 
# 
# # Set A - whole clean Shapefiesl
# gsimA <- gsimClean
# 
# #add on 05.04.2023, remove all missing years before filter in period
# gsimA$year.bg <- gsimA$year.start+gsimA$gap_begin
# gsimA$year.ed <- gsimA$year.end - gsimA$gap_end
# 
# 
# # Set B - only basis having data in period 1960-2010
# gsimB <- gsimA[gsimA$year.bg >= 1960 & gsimA$year.ed <= 2010, ] #11975 basins
# # gsimA1 <- x[x$year.start >= 1960 & x$year.end <= 2010, ] #8417 basins --- the old selection
# 
# writeLines(gsimA[,1], paste0(path,"/gsim.lst"))#, row.names = FALSE, col.names = FALSE)
# gsimLst <- gsimA$gsim.no





