

##=============================================================================================================##
####                           Paper 3 - GLOBAL   xz ANALYSIS                                      ###
##=============================================================================================================##
### Note on 05.09.2019
# The results were re-calculated, because:
#1. The NA value was omitted --> may lead to wrong results. This script interpolate the missing value
#2. Remove all the change points which occrur at the first/last 2 years of results.

lib_lst <- c('changepoint','breakpoint','cpm','trend','climtrends',
             'dplyr','magrittr',
             'rworldmap','tidyverse','foreach')


for (lib in lib_lst){
  if(!require(lib,character.only = TRUE)){install.packages(lib,dependencies = T); library(lib,character.only = TRUE)}}


detectCPT <- function(gsim_lst,
                      Qpath,
                      optSet,
                      flow #= c("MEAN","MIN","MAX","MIN7","MAX7","P10","P90")
                      ){
  # for(flow in flow_lst) {
  z <- list()
    for(i in gsim_lst){
    # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = c("data.table"), .verbose=F) %dopar% {
      ### Print the current ID to follow the process
      cat("Running basin", match(i,gsim_lst), "\n")
      
      #read the manual discharge 
      df <- read.table(paste0(Qpath,'/',i,".year"),header = T, sep = ",")
      
      #convert to numeric
      for(j in 2:ncol(df)) if(!is.numeric(df[, j])) df[,j] <- as.numeric(df[, j])
      
      df$VARB <- df$MAX-df$MIN
      
      #select only the mean discharge to calculate
      df <- df[,c("date",flow)]
      colnames(df)[2] <- 'discharge'
      
      # df$discharge[1:5] <- NA
      
      #----------------- ADDED 14.10.2019 ------------------#
      #consider station having minus discharge --> turn to NA ---> remove
      df[df < 0] <- NA
      #------------------------------------------------------#
      ### ERROR_1: some station has NA discharge
      if(sum(is.na(df$discharge)) > 0.5*length(df$discharge)) {
        res = data.frame(gsim.no = i,
                         Norm=NA,CPTM=NA,BRPT=NA,CUMD=NA,STUT=NA,MWNT=NA,PETT=NA, 
                         len_yrs = paste0('NA_data (', sum(is.na(df$discharge)),')')) 
      } else {
        #interpolate the NA values in files 
        dfz <- read.zoo(df)   #convert to zoo series, prepare for interpolation
        
        #turn the zoo objects to data frame
        dat <- fortify.zoo(na.approx(dfz,na.rm = TRUE))
        names(dat) <- c("date", "discharge")
        
        dat$year <- format(as.Date(dat$date, format="%Y-%m-%d"),"%Y")
        dat <- dat[, c('year','discharge')]
        
        #add on 06.04.2023
        if(grepl('B', optSet)) dat <- dat[dat$year >= 1960 & dat$year <= 2010, ]
        if(grepl('C', optSet)) dat <- dat[min(dat$year) <= 1960 & max(dat$year) >= 2010, ]
        
        
        if (length(dat$discharge) <= 5 | sum(dat$discharge) == 0) {
          res = data.frame(gsim.no = i,
                           Norm=NA,CPTM=NA,BRPT=NA,CUMD=NA,STUT=NA,MWNT=NA,PETT=NA,
                           len_yrs = paste0('NS_data (', length(dat$discharge),')'))
          
        } else {
        
        ### control the class of input data 
        if(class(dat$discharge) == "factor") {
          dat$discharge <- as.numeric(levels(dat$discharge))[dat$discharge] 
          }
        
        dis <- dat$discharge
        len <- length(dis)

          ### ------------------ Normality tests and other change point test ------------------1
          # cat("normality\n")
          nor_test <- shapiro.test(dis)
          if (nor_test$p.value < 0.1) {
            Norm <- NA
          } else {
            Norm <- 'Norm' }
          
          ### ------------------ CPTM ------------------#
          # cat("CPTM\n")
          func <- cpt.mean(dis,penalty="MBIC",pen.value=0.05,method="AMOC",test.stat="Normal") 
          if (func@cpts[1] <=2|func@cpts[1] >len-2) {
            CPTM <- NA 
          } else {
            CPTM <- dat[as.numeric(func@cpts[1]),"year"]}
          
          
          ### ------------------BRPT ------------------#
          # cat("BRPT\n")
          func <-   CE.Normal.Mean(as.data.frame(dis),Nmax = 1,eps = 0.01,rho=0.05,distyp = ,penalty = "mBIC",parallel = FALSE)
          #edit on 06.104.2023
          if(length(func) == 1){
          # if(func == "No Break-Points are Estimated") {
            BRPT = NA
          } else {
            loc <- func$No.BPs
            if(is.na(loc) == TRUE){
              BRPT = NA
            } else {
              if(func$BP.Loc <=2|func$BP.Loc >len-2) {
                BRPT = NA
              } else {
                BRPT = dat[as.numeric(func$BP.Loc),"year"]
              }
            }
          }
          
          
          
          ### ------------------CUMD ------------------#
          # cat("CUMD\n")
          func <- CumulativeDeviations(dis)
          
          #turn to dataframe
          res_df <- data.frame(loc = c(1:length(func)), val = func)
          maxval <- max(res_df$val)
          loc <- res_df[res_df$val==max(res_df$val),"loc"]
          
          if (loc <=2|loc >len-2) { 
            CUMD <- NA
          }  else { 
            loc <- res_df[res_df$val==max(res_df$val),"loc"]
            CUMD <- dat[as.numeric(loc),"year"]
          }
          
          #add on 31.05.2020 ---> xu ly khi co nhieu hon 1 cpt cho max va min
          if(grepl("MIN|MAX|P10",flow) & length(CUMD)>1){ #added on 31.05: thesse stations could  not find cpt for min
            CUMD <- round(median(CUMD),0)
          } 
          
          
          ### ------------------STUT ------------------#
          # cat("STUT\n")
          x <- length(dis[dis>0.0001])  #add on 31.05.2020 due to series of 0.0000 in MIN
          if(grepl("MIN|P10",flow) & i %in% c("AU_0002434","ZA_0000051")){ #added on 31.05: thesse stations could  not find cpt for min
            STUT <- NA
          } else {
            func <- detectChangePointBatch(dis,cpmType = "Student",alpha = 0.05)
            if(func$changePoint == 0) {
              STUT <- NA
            } else {
              STUT <- dat[as.numeric(func$changePoint),"year"]}
          }
          
          
          ### ----------------- MWNT ------------------#
          # cat("MWNT\n")
          func <- detectChangePointBatch(dis,cpmType = "Mann-Whitney",alpha = 0.05)
          if(func$changePoint == 0) {
            MWNT <- NA
          } else {
            MWNT <- dat[as.numeric(func$changePoint),"year"]}
          
          
          ### ----------------- PETT------------------#
          # cat("PETT\n")
          func = pettitt.test(dis)
          if(func$p.value > 0.05) {
            PETT <- NA
          } else {
            PETT <- dat[as.numeric(func$estimate[1]),"year"]}
          
          res = data.frame(gsim.no = i,
                           Norm,CPTM,BRPT,CUMD,STUT,MWNT,PETT,
                           len_yrs = length(dis))
          print(res)
        } #close if-else length
      } #close else for NA data
      
      # return(res)
      z[[i]] <- res
    }# close foreach
    
    require(data.table)
    all_res <- rbindlist(z)
    return(all_res)
  }
# }



# #===================================================================#
# #B --- Fill the results of changes - done  ----'
# #===================================================================#  

finalCPT <- function(cpt_file){
    cpt_all <- read.table(cpt_file,header = TRUE, sep = ",")
    z<- list()
    for(i in 1:nrow(cpt_all)) {
      
      ### Remove all change point from non-normal distribution
      if(is.na(cpt_all$Norm[i]) == TRUE) {
        cpt_all[i,c('CPTM','BRPT','CUMD','STUT')] = NA
      } else { } #close if-else for non-normal distribute
      
      ### Get the vector of detected change point
      x <- as.numeric(cpt_all[i,c('CPTM','BRPT','CUMD','STUT','MWNT','PETT')]); x
      x <- na.omit(x)
      
      num.cpt<- length(x)
      #find the last change point by scanning the 10 years-window
      
      if (num.cpt ==0) {
        cpt_ID <- NA
        diff <- NA
        first = NA ; last = NA
        clas = NA
        
      }else {
        if(num.cpt==1) {
          cpt_ID <- x
          diff <- 0
          first = x ; last = x
          clas = 'weak'
          
        } else {
          first =  min(x); last= max(x)
          diff <- last-first
          
          #scan by 10 years moving
          if(diff <=10){
            cpt_ID = round(mean(x),0)
          } else {
            #scanning 10 years -----------------------------------#
            k <- list()
            for (r in first:last){
              y <- x[x>=r & x< r+10]
              
              k[[i]] = data.frame(gap = paste0(r,"-",r+10),
                                  num = length(y),
                                  cpt_ID = round(mean(y),0))
            }
            res <- do.call(rbind,k)
            
            #select only windows having changepoint
            res <- res[res$cpt_ID != 'NaN'& res$num == max(res$num),]
            
            #find the change points which found the most
            y <- as.data.frame(table(res$cpt_ID)); colnames(y) <- c('year','Freq')
            cpt_ID <- as.numeric(as.character(y$year[y$Freq==max(y$Freq)]))
            #----------------------------------------------------#
            #----------- classify the changes by strength of agreement by the tests -----------------------#
          }#close for more than 10 years different
          ifelse(num.cpt >= 3 & diff <=10,
                 clas<- "strong",
                 clas <- 'pending')
        }#close for 1cpt
      }#close for 0 cpt
      
      
      ### get them all into dataframe
      res <- data.frame(gsim.no = cpt_all$gsim.no[i],
                        num.cpt,
                        first,
                        last,
                        diff,
                        cpt_ID,
                        cpt_clas=clas)
      z[[i]] <- res
      
    }#close foreach
    
    res <- do.call(rbind,z)
    
    library(plyr)
    res_join <- full_join(cpt_all,res,by='gsim.no')
    return(res_join)
    }



# IMPORTANT - most of data process in this function are based on func detectCPT
detectTrend <- function(gsim_lst,
                        Qpath,
                        optSet,
                        flow 
                        ){
  z <- list()
  for(i in gsim_lst){
    # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = c("data.table"), .verbose=F) %dopar% {
    ### Print the current ID to follow the process
    cat("Running basin", match(i,gsim_lst), "\n")
    
    #read the manual discharge 
    df <- read.table(paste0(Qpath,'/',i,".year"),header = T, sep = ",")
    
    #convert to numeric
    for(j in 2:ncol(df)) if(!is.numeric(df[, j])) df[,j] <- as.numeric(df[, j])

    
    #select only the mean discharge to calculate
    df <- df[,c("date",flow)]
    colnames(df)[2] <- 'discharge'
    
 
    #----------------- ADDED 14.10.2019 ------------------#
    #consider station having minus discharge --> turn to NA ---> remove
    df[df < 0] <- NA
    #------------------------------------------------------#
    ### ERROR_1: some station has NA discharge
    if(sum(is.na(df$discharge)) > 0.5*length(df$discharge)) {
      res <- data.frame(gsim.no = i,
                        z = NA,
                        S = NA,
                        varS = NA,
                        tau = NA,
                        p.value =  NA,
                        significance = NA
      )
    } else {
      #interpolate the NA values in files 
      dfz <- read.zoo(df)   #convert to zoo series, prepare for interpolation
      
      #turn the zoo objects to data frame
      dat <- fortify.zoo(na.approx(dfz,na.rm = TRUE))
      names(dat) <- c("date", "discharge")
      
      dat$year <- format(as.Date(dat$date, format="%Y-%m-%d"),"%Y")
      dat <- dat[, c('year','discharge')]

      #add on 06.04.2023
      if(grepl('B', optSet)) dat <- dat[dat$year >= 1960 & dat$year <= 2010, ]
      if(grepl('C', optSet)) dat <- dat[min(dat$year) <= 1960 & max(dat$year) >= 2010, ]
      
      if (length(dat$discharge) <= 10 | sum(dat$discharge) == 0) {
        res <- data.frame(gsim.no = i,
                          z = NA,
                          S = NA,
                          varS = NA,
                          tau = NA,
                          p.value =  NA,
                          significance = NA
        )
        
      } else {
        
        ### control the class of input data 
        if(class(dat$discharge) == "factor") {
          dat$discharge <- as.numeric(levels(dat$discharge))[dat$discharge] 
        }
        
        dis <- dat$discharge
        
        mk <- trend::mk.test(dis, alternative = "two.sided")

        p <- mk$p.value
        sig <- ifelse(p < 0.1, "Sig", "nonSig")
        S <- mk$estimates[1]
        change <- ifelse(S >0, "In", "De")

        res <- data.frame(gsim.no = i,
                          z = mk$statistic,
                          S,
                          varS = mk$estimates[2],
                          tau = mk$estimates[3],
                          p.value = p,
                          significance = sig,
                          change,
                          trend = paste0(sig, "_", change)
                          )
        print(res)
      } #close if-else length
    } #close else for NA data
    
    # return(res)
    z[[i]] <- res
  }# close foreach
  
  require(data.table)
  all_res <- rbindlist(z, fill = TRUE)
  return(all_res)
}







