


lib_lst <- c('sp','maptools','raster','foreach','doParallel','data.table','sf')
for (lib in lib_lst){
  if(!require(lib,character.only = TRUE)){install.packages(lib,dependencies = T); library(lib,character.only = TRUE)}}


#++++++++++++++++++++++++++++++++++ EXTRACT ENVIRONMENTAL VARIABLE  ++++++++++++++++++++++++++++++++++++++++++#

#rewrite due re-classification LC directly inside code
#this function works only with Landcover in D:\03. Work\00. PhD\00. Database\GlobalAnalysis\Landcover\outTiffs_PrePost
# extVar <- function(out_dir,
#                    dataSet,
#                    flow,
#                    VAR,
#                    GlobalDATA,
#                    RDATADIR
#                    ){

extVar <- function(gsim_cpt,
                   VAR, #c('Temperature', 'Precipitation','EvapoTrans','Landcover','WaterUse')
                   GlobalDATA,
                   RDATADIR){

  gsim_lst <- as.character(gsim_cpt$gsim.no)
  noRdata_files <- readLines(paste0(path,"/noRdata.lst"))
  
  #get directory
  VAR_DIR <- paste0(GlobalDATA,'/',VAR,'/outTiffs'); VAR_DIR
  
  if(grepl("Precipitation", VAR)) VAR_DIR <- paste0(GlobalDATA,'/',ivar,'/outTiffs'); VAR_DIR
  if(grepl("WaterUse", VAR)) VAR_DIR <- paste0(GlobalDATA,'/',VAR,'/outTiffs_',ivar); VAR_DIR
  if(grepl("Landcover", VAR)) VAR_DIR <- paste0(GlobalDATA,'/',VAR,'/outTiffs_PrePost/outTiffs_',ivar); VAR_DIR #landcover will no more run in this script
  if(grepl("EvapoTrans|WaterUse|wateruse|Wateruse", VAR)) VAR_DIR <- paste0(GlobalDATA,'/',VAR,'/outTiffs_',ivar); VAR_DIR
  
  if(!file.exists(VAR_DIR)) stop(paste0(VAR_DIR, " not found!"))
  
  cat("*** starting for",VAR,':',ivar,'...\n')
  
  #check avaialbe tiff files
  tiff.files <- setdiff(list.files(VAR_DIR), list.dirs(recursive = FALSE, full.names = FALSE))
  
  z <- lapply(tiff.files, function(x) 
    strsplit(strsplit(x, "_")[[1]][1], '_')[[1]][1])
  
  z <- as.numeric(do.call(rbind,z))
  
  tiff_start <- min(z) 
  tiff_end <- max(z) 
  
  # i <- "BR_0000050"
  ### !!!!!!!!!!!!!! START CALCULATING FROM HERE !!!!!!!!!!!!!!!!!!!!!!!!!!
  # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = c("raster", "data.table"), .verbose=F) %dopar% {
  z <- list()
  for(i in gsim_lst){
    cat("running",paste0(VAR,'_',ivar,":"), match(i,gsim_lst), "\n")
    cpt_ID <- gsim_cpt[match(i,gsim_cpt$gsim.no),'cpt_ID']  #--> return the NA, otherwise integer(0)
    
    
    ### (ERROR_1): Check the significant change point detected for basin
    if(is.na(cpt_ID) == TRUE) { #-----------------------------No significant change point detected --> return to NA
      # print('NA - No changepoint detected')
      myres <- data.frame(gsim.no=i,
                          cpt_ID = NA,
                          pre_avg=NA, post_avg=NA)#,
      #  pre_sum=NA, post_sum=NA)
      myres$note <- 'NS_cpt'  #'Non-sig changepoint'
    } else {
      
      ### (ERROR_2) Check the .Rdata files are available for basin
      if (i %in% noRdata_files) {  #----------------------------- No Rdata files/shp available, return to NA resuls
        # print('NA - Rdata files not available')
        myres <- data.frame(gsim.no=i,
                            cpt_ID,
                            pre_avg=NA, post_avg=NA)#,
        #  pre_sum=NA, post_sum=NA)
        myres$note <- 'NA_shps'  #'Non-sig changepoint'
      } else {  
        
        ### Get the basin shape (pre-cooked as Rdata file to speed up)
        load(paste0(RDATADIR, "/",  gsub(".shp", "",i), ".RData")) # tmp_shp, has FILENAME attribute as id
        # tmp_shp <- shapefile(paste0(SHAPEDIR, '/',i, '.shp'))     #this is original one without pre-cooked Rdata files
        
        ### (ERROR_3): Check the if the global Tiff provided for the detected changepoint
        if(cpt_ID < tiff_start | cpt_ID > tiff_end) {  #----------No available outTiff changes for the corresponding cpt
          # print('NA - OutTiffs not found')
          myres <- data.frame(gsim.no=i,
                              cpt_ID,
                              pre_avg=NA, post_avg=NA)#,
          #pre_sum=NA, post_sum=NA)
          myres$note <- 'NA_outTiffs'  #'OutTiffs not found'
          
        } else {      #----------------------------------------- Start calculating changes for each basin
          ### Call the global rater/tif files of the corresponding pre/post cpt_ID 
          
          # FLIP the raster by y-direction (if this is landcover)
          if(VAR == 'Landcover'| VAR == 'WaterUse') {
            r1 <- flip(raster(paste0(VAR_DIR, "/",cpt_ID,"_prior.tif")),direction = 'y')
            r2 <- flip(raster(paste0(VAR_DIR, "/",cpt_ID,"_post.tif")),direction = 'y')
          } else {
            r1 <- raster(paste0(VAR_DIR, "/",cpt_ID,"_prior.tif"))
            r2 <- raster(paste0(VAR_DIR, "/",cpt_ID,"_post.tif"))
          }#close if-else for flipping map of landcover
          
          # plot(r1) ; plot(tmp_shp, col="red", add=TRUE)  #add shapefile
          
          ### stack 2 files
          s <- stack(r1, r2)
          names(s) <- c('pre','post')
          
          ### crop the changes (around the globe) to the actual .shp of the stations
          sc <- crop(s, extent(tmp_shp)) ; sc
          
          ### (ERROR_4): Check the shapefiles can be used or throw an error - if this happens, write "NA" to enable filtering later on
          try(myres_avg <- as.data.frame(raster::extract(sc, tmp_shp, fun=mean, na.rm=T)), silent = T)
          # try(myres_sum <- as.data.frame(raster::extract(sc, tmp_shp, fun=sum, na.rm=T)), silent = T)
          
          if(exists("myres_avg")) {
            names(myres_avg) <- c('pre_avg','post_avg')
            # names(myres_sum) <- c('pre_sum','post_sum')
            
            myres <- data.frame(gsim.no = i,
                                cpt_ID)
            myres <- cbind(myres,myres_avg)#, myres_sum)
            myres$note <- '-'
            
            
          } else {
            # print('NA - Not proper shapefiles found')
            myres <- data.frame(gsim.no=i,
                                cpt_ID,
                                pre_avg=NA, post_avg=NA)#,
            #  pre_sum=NA, post_sum=NA)
            myres$note <- 'NA_shps' #'Error in shps'
            
          }#close the if-else for error of shp file
        }#close the if-else for the unavailable Outtiffs 
      }#close loop for noRdata_files
    }#close the if-else for NA changepoint value
    
    z[[i]] <- myres
  }# close foreach
  require(data.table)
  all_res <- rbindlist(z) 
  
  return(all_res)
}#end function



#++++++++++++++++++++++++++++++++++ Precook Landcover (5 class) ++++++++++++++++++++++++++++++++++++++----
#This is to extract whole 5 new classifications of LC sent by Jens on 09.04.2023 to each year (1960.Tiffres)

precookLC <- function(gsim_lst, #list of gsim ID
                      year_lst, #year we want to extract for whole gsim list
                      tif_files, #all tif files to extract values (get from Jens)
                      RDATADIR # shapefile data of each gsim basin
                      ){
  z <- list()
  for(i in gsim_lst){
    
    #get shapefiles of basins
    load(paste0(RDATADIR, "/",  gsub(".shp", "",i), ".RData"))#; plot(tmp_shp)
    
    zy <- list()
    for(y in year_lst){
      tif_file <- tif_files[grepl(y, tif_files)]
      
      zf <- list()
      for(tif in tif_file){
        cat("Running basin", match(i,gsim_lst), "in", y, "\n")
        # load raster of LC
        r <- raster::raster(tif)#; plot(r)
        # plot(r) ; plot(tmp_shp, col="red", add=TRUE)  #add shapefile
        
        ### crop the changes (around the globe) to the actual .shp of the stations
        sc <- raster::crop(r, extent(tmp_shp)) 
        
        #extract values 
        try(val_res <- as.data.frame(raster::extract(sc, tmp_shp, fun = mean, na.rm=T)), silent = T)
        
        zf[[tif]] <- val_res$V1
      }#end for file
      res <- as.data.frame(do.call(cbind, zf))
      
      for(j in 1:ncol(res)){
        on <- colnames(res)[j]
        nn <- strsplit(on, "/")[[1]][8]
        colnames(res)[j] <- nn
      }
      
      res <- cbind(y, res)
      zy[[y]] <- res
    }#end for year
    
    res <- rbindlist(zy)
    z[[i]] <- cbind(i, res)
  }#end for gsim
  res <- rbindlist(z)
  colnames(res)[1:2] <- c("gsim.no", "year")
  
  res$sum <- NA
  for(i in 1:nrow(res)){
    res$sum[i] <- res$Cropland[i] + res$Forest[i] + res$Grassland[i] + res$NoVegetation[i] + res$Urbanland[i]
  }
  
  return(res)
}#end func

#++++++++++++++++++++++++++++++++++ PRE/POST Landcover ++++++++++++++++++++++++++++++++++++++----
#this function calculate pre/post change of LC based on new classification from Jens

extrLCnew <- function(gsim_lst, #list of gsim_list
                      gsim_cpt, # file contains CPT
                      LC, #landcover Cropland, Grassland....
                      precookLC_dir #path to precooked LC
                      ){
  
  LC_files <- list.files(precookLC_dir, 
                         pattern = ".Tiffres", full.names = T)
  z <- list()
  for(i in gsim_lst){
    cat("...", LC, "in", i, " -", match(i,gsim_lst),"\n")
    
    zf <- list()
    for(f in LC_files){
      df <- fread(f)
      df <- df[df$gsim.no == i,]#; print(head(df))
      zf[[f]] <- df
    }
    res <- as.data.frame(rbindlist(zf))
    
    #get cpt
    cpt_ID <- gsim_cpt[match(i,gsim_cpt$gsim.no),'cpt_ID']
    
    if(is.na(cpt_ID)){ #when no changepoint --> no changes defines
      pre_avg = NA
      post_avg = NA
    } else{            #when with cpt --> then define changes
      res <- res[, c("year", LC)]; names(res)[2] <- "tempvar"
      pre_avg <- mean(res$tempvar[res$year < cpt_ID])
      post_avg <- mean(res$tempvar[res$year >= cpt_ID])
      
    }
    myres <- data.frame(gsim.no = i,
                        cpt_ID,
                        pre_avg,
                        post_avg)
    
    z[[i]] <- myres
  }
  
  res_all <- rbindlist(z)
  return(res_all)
}#end function



#++++++++++++++++++++++++++++++++++ EXTRACT RANDOM VALUE ++++++++++++++++++++++++++++++++++++++#
extRandVar <- function(gsim_cpt,
                       GlobalDATA,
                       RDATADIR
                       ){
  gsim_lst <- as.character(gsim_cpt$gsim.no)
  noRdata_files <- readLines(paste0(path,"/noRdata.lst"))
  
  #+++++++++++++++++++++++++++++++++++++++       START CALCULATING       ++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  VAR_DIR <- paste0(GlobalDATA,'/RandomVar')
  
  ### !!!!!!!!!!!!!! START CALCULATING FROM HERE !!!!!!!!!!!!!!!!!!!!!!!!!!
  z <- list()
  for (i in gsim_lst){
    # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = c("raster", "data.table"), .verbose=F) %dopar% {
    cat(paste0(i,' (',match(i,gsim_lst),")"),'\n')
    cpt_ID <- gsim_cpt[match(i,gsim_cpt$gsim.no),'cpt_ID']  #--> return the NA, otherwise integer(0)
    
    
    ### (ERROR_1): Check the significant change point detected for basin
    if(is.na(cpt_ID) == TRUE) { #-----------------------------No significant change point detected --> return to NA
      # cpt_ID = NA
      pre_avg=NA
      post_avg=NA
      note <- 'NS_cpt'  #'Non-sig changepoint'
      
    } else {
      
      ### (ERROR_2) Check the .Rdata files are available for basin
      if (i %in% noRdata_files) {  #----------------------------- No Rdata files/shp available, return to NA resuls
        pre_avg=NA
        post_avg=NA
        note <- 'NA_shps'  #'Non-sig changepoint'
        
      } else {  
        
        ### Get the basin shape (pre-cooked as Rdata file to speed up)
        load(paste0(RDATADIR, "/",  gsub(".shp", "",i), ".RData")) # tmp_shp, has FILENAME attribute as id
        
        r1 <- raster(paste0(VAR_DIR, "/",cpt_ID,"_prior.tif"))
        r2 <- raster(paste0(VAR_DIR, "/",cpt_ID,"_post.tif"))
        
        # plot(r2); plot(tmp_shp, col="red", add=TRUE)  #add shapefile
        # plot(tmp_shp)
        t <- extent(tmp_shp)
        # add if not overlap extend
        ex <- data.frame(r1 = c(r1@extent@xmin,r1@extent@xmax,r1@extent@ymin,r1@extent@ymax),
                         r2 = c(r2@extent@xmin,r2@extent@xmax,r2@extent@ymin,r2@extent@ymax),
                         t = c(t@xmin,t@xmax,t@ymin,t@ymax))
        
        for(j in 1:nrow(ex)){
          if(j==1|j==3){
            ex$overlap[j] <- ifelse(ex$t[j] < ex$r1[j] | ex$t[j] < ex$r2[j],
                                    "x",
                                    "-")
          } else{
            ex$overlap[j] <- ifelse(ex$t[j] > ex$r1[j] | ex$t[j] > ex$r2[j],
                                    "x",
                                    "-")
          }
        }
        
        if("x" %in% ex$overlap){
          cat('extent not overlap. Default value returned \n')
          pre_avg = 500
          post_avg = 500
        } else {
          s1 <- crop(r1, extent(tmp_shp))
          pre_avg <- raster::extract(s1, tmp_shp, fun=mean, na.rm=T)
          
          
          s2 <- crop(r2, extent(tmp_shp))
          post_avg <- raster::extract(s2, tmp_shp, fun=mean, na.rm=T)
        }
        
        note <- "-"
      }
    }
    # return(myres)
    z[[i]] <-  data.frame(gsim.no = i,
                          cpt_ID,
                          pre_avg,
                          post_avg,
                          note)
  }

  all_res <- do.call(rbind,z)   # all_res <- rbindlist(results)
  return(all_res)
}


#++++++++++++++++++++++++++++++++++ Process discharge ++++++++++++++++++++++++++++++++++++++----
#used for extDis() below
require(zoo)
processQ <- function(Qfile,
                     flow,
                     dataSet){
  
  #read the manual discharge 
  df <- read.table(Qfile, header = T, sep = ",")
  
  #convert to numeric
  for(j in 2:ncol(df)) if(!is.numeric(df[, j])) df[,j] <- as.numeric(df[, j])
  df$VARB = df$MAX-df$MIN
  
  #select only the mean discharge to calculate
  df <- df[,c("date",flow)]
  colnames(df)[2] <- 'discharge'
  
  #consider station having minus discharge --> turn to NA ---> remove
  df[df < 0] <- NA
  
  #interpolate the NA values in files 
  dfz <- zoo::read.zoo(df)   #convert to zoo series, prepare for interpolation
  
  #turn the zoo objects to data frame
  dat <- zoo::fortify.zoo(na.approx(dfz,na.rm = TRUE))
  names(dat) <- c("date", "discharge")
  
  dat$year <- format(as.Date(dat$date, format="%Y-%m-%d"),"%Y")
  dat <- dat[, c('year','discharge')]
  
  #select dataset by option
  if(grepl('A', dataSet)) dat <- dat
  if(grepl('B', dataSet)) dat <- dat[dat$year >= 1960 & dat$year <= 2010, ]
  if(grepl('C', dataSet)) dat <- dat[min(dat$year) <= 1960 & max(dat$year) >= 2010, ]
  
  return(dat)
  
}

#++++++++++++++++++++++++++++++++++ EXTRACT DISCHARGE  ++++++++++++++++++++++++++++++++++++++----
#need function processQ() above
extrDis <- function(gsim_cpt,
                    Qpath
                    ){

  gsim_lst <- as.character(gsim_cpt$gsim.no)

  z <- list()
  for(i in gsim_lst){
  # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = "data.table", .verbose=F) %dopar% {

    cat("Running basin", match(i,gsim_lst), "\n")
    
    ### Get the change point year (cpt_ID) of each station
    cpt_ID <- gsim_cpt[match(i,gsim_cpt$gsim.no),'cpt_ID']  #--> return the NA, otherwise integer(0)
    
    if(is.na(cpt_ID) == TRUE) { #-----------------------------No significant change point detected --> return to NA
      print('NA - No changepoint detected') 
      res <- data.frame(gsim.no = i,
                        cpt_ID, 
                        pre_avg = NA, post_avg = NA)
    } else {### add the mean discharge before/after change point
      df <- processQ(Qfile = paste0(Qpath,"/",i,".year"),
                     flow,
                     dataSet)

      if(sum(is.na(df$discharge)) > 0.5*length(df$discharge)) {
        print(paste0('NS - Not Sufficient data: ',sum(is.na(df$discharge)), '/',length(df$discharge),' missing')) 
        res <- data.frame(gsim.no = i,
                          cpt_ID, 
                          pre_avg = NA, post_avg = NA)
        
      } else {
        res <- data.frame(gsim.no = i,
                          cpt_ID, 
                          pre_avg = mean(df[df$year< cpt_ID,'discharge']),
                          post_avg = mean(df[df$year>= cpt_ID,'discharge']))
      }#close for too long missing value
    }#close for NA changepoint
    z[[i]] <- res
  }# close foreach/for
  all_res <- do.call(rbind,z)   # all_res <- rbindlist(results)
  return(all_res)
}#end of function


#++++++++++++++++++++++++++++++++++ EXTRACT CAPACITY OF DAM ++++++++++++++++++++++++++++++++++++++----

# writen on 09.04.2023
# extract capacity of dam for each basins, which is:
# based on GRAND_FULL_list (see PreCook)
# base on Damset
extDamCAP <- function(gsim_lst, #list of gsim.basins (as character)
                      damSet, #select dam built in certain period
                      grand,  #fulldam list of Grand (see precook)
                      opt #option to extract full (Full), NW (no withdraw) and W (withdraw)
                      ){
  z <- list()
  for(i in gsim_lst){
    cat("running",opt, "---", match(i, gsim_lst), i,"\n")
    df <- grand[grand$gsim.no == i, ]
    
    #remove dams having no YEAR, as we need to calculate changes based on year
    df<- df[!is.na(df$YEAR), ]
    
    # consider only dams build in certain period 
    if(grepl("B", damSet)) df <- df[df$YEAR >= 1960 & df$YEAR <= 2010,] #within period 1961-2010
    if(grepl("C", damSet)) df <- df[df$YEAR < 1960,] #only dams build in period before 1960
    if(grepl("D", damSet)) df <- df[df$YEAR < 2010,] #only dams build until 2010
    
    #subset by purpose
    if(opt == "W") df <- df[df$MAIN_USE %in% c('Irrigation','Water supply'),]
    if(opt == "NW") df <- df[!df$MAIN_USE %in% c('Irrigation','Water supply'),]
    
    #get CPT
    cpt_ID <- gsim_cpt$cpt_ID[match(i ,gsim_cpt$gsim.no)] 
    
    #find first and last built dam
    num.dam <- length(df$GRAND_ID); num.dam
    if(is.na(num.dam) | num.dam == 0 | is.na(cpt_ID)){
      res <- data.frame(gsim.no = i,
                        cpt_ID,
                        dam_num = 0,
                        first_dam = 0, #before was NA, changed on 23.04.2023 
                        last_dam = 0, #before was NA, changed on 23.04.2023
                        dam_pre = 0,
                        dam_post = 0,
                        cap_pre  = 0, #before was NA, changed on 23.04.2023
                        cap_post = 0 #before was NA, changed on 23.04.2023
      )
    } else {
      res <- data.frame(gsim.no = i,
                        cpt_ID,
                        dam_num = num.dam,
                        first_dam = min(df$YEAR),
                        last_dam = max(df$YEAR),
                        dam_pre = nrow(df[df$YEAR<cpt_ID,]),
                        dam_post = nrow(df[df$YEAR>=cpt_ID,]),
                        cap_pre  = sum(df$CAP_MCM[df$YEAR<cpt_ID]),
                        cap_post = sum(df$CAP_MCM)
      ); res
    }
    z[[i]] <- res
  }
  res <- rbindlist(z)
}






