



source(paste0(path,"/_supportFuncs/Func00_General.R"))

lib_lst <- c('dplyr',"data.table") 

for (lib in lib_lst){
  if(!require(lib,character.only = TRUE)){install.packages(lib,dependencies = T); library(lib,character.only = TRUE)}}


sum_by_row <- function(df){
  df <- cbind(df, sum = rowSums(df))
  #return as a vector
  #df <- rowSums(df,na.rm = TRUE)*ifelse(rowSums(is.na(df))==ncol(df),NA,1)
  return(df)
}


cbineForest <- function(VARlist,
                        period,
                        out_tiff,
                        out_tiff_clean
                        ){
  for(VAR in VARlist){
    cat(VAR,'\n')
    ivar_lst <- switch(VAR,
                       NatureForest = paste0("Landcover_",c('TrpEBF','TrpDBF','TmpEBF','TmpENF','TmpDBF','BorENF','BorDNF')),
                       SecondaryForest = paste0("Landcover_",c('SecTrpEBF','SecTrpDBF','SecTmpEBF','SecTmpENF','SecTmpDBF','SecBorENF','SecBorDNF')),
                       Shrubland = paste0("Landcover_",c('Openshrub', 'Denseshrub')))
    
    z <- list()
    for (p in period) {
      
      k <- list()
      for (iVAR in ivar_lst){
        cat(iVAR,'\n')
        
        iVAR_DIR <- paste0(out_tiff,'/',iVAR,'.Rres'); iVAR_DIR  
        df <- read.table(iVAR_DIR,header = TRUE,sep = ",")
        x <- df[colnames(df)%in% p ] 
        
        k[[iVAR]] <- x
        
      }#close loop for all
      
      res <- do.call(cbind,k)
      res$sum <- rowSums(res)  #when error, use  sum_by_row(res)
      
      z[[p]] <- res$sum
    }
    res <- do.call(cbind,z) 
    res <- cbind(df[,c('gsim.no','cpt_ID')],res)
    
    if(!exists(out_tiff_clean)){dir.create(out_tiff_clean)}
    
    out_file <- paste0(out_tiff_clean,'/',VAR,'.Rres')
    write.table(res,out_file, row.names = FALSE, sep = ",") 
  }#end for VAR
}#end of function




cbineForestNEW <- function(VARlist,
                           flow,
                           period,
                           out_tiff,
                           out_tiff_clean
                           ){
  for (flow in flow_lst){
    for(VAR in VARlist){
      cat(VAR,'\n')
      ivar_lst <- switch(VAR,
                         Forest = paste0("Landcover_",c('TrpEBF','TrpDBF','TmpEBF','TmpENF','TmpDBF','BorENF','BorDNF',
                                                        'SecTrpEBF','SecTrpDBF','SecTmpEBF','SecTmpENF','SecTmpDBF','SecBorENF','SecBorDNF')),
                         Shrubland = paste0("Landcover_",c('Openshrub', 'Denseshrub')))
      
      z <- list()
      for (p in period) {
        
        k <- list()
        for (iVAR in ivar_lst){
          cat(iVAR,'\n')
          
          iVAR_DIR <- paste0(out_tiff,'_',flow,'/',iVAR,'.Rres'); iVAR_DIR  
          df <- read.table(iVAR_DIR,header = TRUE,sep = ",")
          x <- df[colnames(df)%in% p ] 
          
          k[[iVAR]] <- x
          
        }#close loop for all
        
        res <- do.call(cbind,k)
        res$sum <- rowSums(res)  #when error, use  sum_by_row(res)
        
        z[[p]] <- res$sum
      }
      res <- do.call(cbind,z) 
      res <- cbind(df[,c('gsim.no','cpt_ID')],res)
      
      out_res_clean <- paste0(out_tiff_clean,'_',flow)
      if(!exists(out_res_clean)) dir.create(out_res_clean)
      
      out_res <- paste0(out_res_clean,'/',VAR,'.Rres')
      write.table(res,out_res, row.names = FALSE, sep = ",") ##write results to tile
    }#end for VAR
  }#end for flow
}#end of function





#=========================  C3,C4 Grass, cropland, pastureland ====================================#
cbineC3C4 <- function(VARlist,
                      Clist,
                      flow,
                      period,
                      out_tiff,
                      out_tiff_clean
                      ){
  for (flow in flow_lst){
    
    y <- list(); c <- 'past'
    for (c in VARlist) {
      
      z <- list()
      for (p in period) {
        
        k <- list(); i <- 'C3'
        for (i in Clist){
          VAR <- paste0('Landcover_',i,c)
          cat(VAR,'\n')
          
          VAR_DIR <- paste0(out_tiff,'_',flow,'/',VAR,'.Rres')
          
          df <- as.data.frame(fread(VAR_DIR))
          x <- df[colnames(df)%in% p ] 
          
          k[[i]] <- x
          
        }#close loop for all
        
        res <- do.call(cbind,k)
        z[[p]] <- rowSums(res,na.rm = TRUE)*ifelse(rowSums(is.na(res))==ncol(res),NA,1) #apply(res,1,sum)
      }
      res <- do.call(cbind,z) 
      res <- cbind(df[,c('gsim.no','cpt_ID')],res)
      
      #rename to be easy to call
      cp <- switch(c,
                   grass = 'Grassland',
                   crop =  'Cropland',
                   past = 'Pastureland')

      out_res <- paste0(out_tiff_clean,'_',flow,'/',cp,'.Rres')
      write.table(res,out_res, row.names = FALSE, sep = ",") ##write results to tile
    }#end for VAR
  }#end for flow
}#end of function





#=========================  NO VEGETATION ====================================#

#here wer have 2 ways to group them to one
#1-summary of all the value from original files  --> there may have many basin has >90% of desert ---> not explainable
#2-they are calculatef by minus 100 for the current one --> IN USE

noVeg <- function(flow,
                  option, #to define 
                  period,
                  lc_listed,
                  lc_rest,
                  out_tiff,
                  out_tiff_clean
                  ){
  
  for (flow in flow_lst){
    if(option==1) {

      z <- list()
      for (p in period) {
        print(p)
        k <- list()
        for (VAR in lc_rest){
          VAR_DIR <- paste0(out_tiff,'_',flow,'/',VAR,'.Rres'); VAR_DIR 

          ### Get the pre- and post-changes 
          df <- read.table(VAR_DIR,header = TRUE,sep=',')
          df <- df[colnames(df) %in% c('gsim.no','pre_avg','post_avg')]
          
          #combine with continent
          df <- full_join(df,gsim[,c('gsim.no','continent')],'gsim.no')
          
          
          ### --------------- Summary by continents ---------------------------#
          cont_lst <- levels(df$continent); c=cont_lst[2]
          
          z1<-list()
          for (c in cont_lst){
            dfc <- df[df$continent==c,]
            y <- dfc[!is.na(dfc$pre_avg),]
            
            r <- data.frame(cont=c,med_pre = median(y$pre_avg),med_post=median(y$post_avg),
                            avg_pre = mean(y$pre_avg),avg_post=mean(y$post_avg),
                            max_pre = max(y$pre_avg),max_post=max(y$post_avg),
                            q_pre =quantile(y$pre_avg,prob=.975))
            z1[[c]] <- r
          }
          res <- do.call(rbind,z1)
          #----------------------------------------------------------------------#
          
          colnames(df)[2:3] <- c('Pre','Post')
          
          #select by period
          k[[VAR]] <- df[colnames(df) %in% p]
        }#close loop for all
        res <- as.data.frame(do.call(cbind,k))
        
        colnames(res) <- lc_rest
        
        #group them all to one
        res<- sum_by_row(res)  #rowSums(res,na.rm = TRUE)*ifelse(rowSums(is.na(res))==ncol(res),NA,1)
        
        # res$NoVegetation <- ifelse(res$sum>100,0,100-res$sum)
        
        z[[p]] <- res$sum #res$NoVegetation
      }
      
      res <- as.data.frame(do.call(cbind,z))
      
      colnames(res) <- c('pre_avg','post_avg')
      res$gsim.no <- df$gsim.no
      
      #write to disk to compare
      write.table(res,paste0(out_dir,'/Results_Precook/NoVeg_Sum_',flow,'.csv'),row.names = FALSE,sep = ',') #to compare
      
      #---------------------------------------------------------------------------------#
    } else {#the NoVegetation area is calculated by minus 100-current area
      #list all the current landcover area (see )
      period_lst <- c('Pre','Post') ; p <- 'Pre'
      z <- list()
      for (p in period_lst) {
        print(p)
        k <- list()
        for (VAR in lc_listed){
          ### Print the current ID to follow the process
          cat("Running for",VAR, "\n") # match(i,Rres.files)
 
          VAR_DIR <- paste0(out_tiff_clean,'_',flow,'/',VAR,'.Rres'); VAR_DIR  
          
          ### Get the pre- and post-changes 
          df <- read.table(VAR_DIR,header = TRUE,sep=',')
          df <- df[colnames(df) %in% c('gsim.no','pre_avg','post_avg')]
          
          colnames(df)[2:3] <- c('Pre','Post')
          
          k[[VAR]] <- df[colnames(df) %in% p]
        }#close loop for all
        res <- as.data.frame(do.call(cbind,k))
        colnames(res) <- lc_listed
        
        #group them all to one
        res <- sum_by_row(res) #rowSums(res,na.rm = TRUE)*ifelse(rowSums(is.na(res))==ncol(res),NA,1)
        # res$NoVegetation <- ifelse(res$sum>=100,0,100-res$sum)
        z[[p]] <- 100-res$sum #res$NoVegetation
      }
      
      res <- as.data.frame(do.call(cbind,z)); colnames(res) <- c('pre_avg','post_avg')
      res$gsim.no <- df$gsim.no
      
      #write to disk to compare
      write.table(res,paste0(out_dir,'/Results_Precook/NoVeg_100_',flow,'.csv'),row.names = FALSE,sep = ',') #to compare
      
    }#close for if-else
    res <- res[,c('gsim.no','pre_avg','post_avg')]
    
    
    out_res <- paste0(out_tiff_clean,"_",flow,'/NoVegetation.Rres')
    write.table(res,out_res, row.names = FALSE, sep = ",") ##write results to tile
    
  }#end for flow
}#end of function




#add on 22.01.2021 --> reduce number of landcover
cbineLAND <- function(land.list,
                      out_tiff_clean
                      ){
  
  for(flow in flow_lst){
    out_tiff_clean_flow <- paste0(out_dir,'/Results_outTiffsRres/outTiffsRres_clean_',flow)
    
    for(land in land.list){
      print(land)
      files <- list.files(out_tiff_clean_flow,".Rres",full.names = TRUE)
      
      select <- switch(land,
                       Forest = c('NatureForest','SecondaryForest'),
                       Urbanland = c('Cropland','Urbanland'),
                       Grassland = c('Grassland','Pastureland'),
                       NoVegetation = c('Shrubland','NoVegetation'))
      
      files <- files[as.numeric(lapply(select,function(x) grep(x,files)))]
      
      z1 <- lapply(c('pre_avg','post_avg'), function(j){
        z <- lapply(files,function(file){
          df <- read.table(file,header = TRUE,sep = ',')
          df <- df[,c('gsim.no',j)]
        })
        res <- as.data.frame(do.call(cbind,z))
        res <- res[,-3]
        
        res$sum <- as.numeric(lapply(1:nrow(res),function(i){
          val <- as.numeric(res[i, -which(names(res) %in% 'gsim.no')])
          val <- sum(val)
        }))
        res <- res[,c('gsim.no','sum')]
        names(res)[2] <- j
        res
      })
      res <- do.call(cbind,z1)
      res <- res[,-3]
      
      file.remove(files)
      write.table(res,
                  paste0(out_tiff_clean_flow,'/',land,'.Rres'), row.names = FALSE, sep = ",") ##write results to tile
    }
  }
}#close function



#add on 22.01.2021 --> reduce number of landcover
cbineLANDnew <- function(land.list,
                         out_tiff_clean
){
  
  for(flow in flow_lst){
    out_tiff_clean_flow <- paste0(out_dir,'/Results_outTiffsRres/outTiffsRres_clean_',flow)
    
    for(land in land.list){
      print(land)
      select <- switch(land,
                       Forest = c('NatureForest','SecondaryForest'),
                       Urbanisation = 'Urbanland',
                       Agriculture= c('Grassland','Pastureland','Cropland'),
                       NoVegetation = c('Shrubland','NoVegetation'))
      files <- list.files(out_tiff_clean_flow,".Rres",full.names = TRUE)
      # grepl(paste(select, collapse = "|"), basename(files))
      
      files<- files[as.numeric(lapply(select,function(x) grep(x,files)))]
  
      if(grepl("Urban",land)){
        res <- read.table(files,header = TRUE,sep = ',')
        res <- res[,c('gsim.no','pre_avg','post_avg')]
      } else {
        z1 <- lapply(c('pre_avg','post_avg'), function(j){
          z <- lapply(files,function(file){
            df <- read.table(file,header = TRUE,sep = ',')
            df <- df[,c('gsim.no',j)]
          })
          res <- cbindNEW(z)
          res <- removedoubleColnames(res,'gsim.no')
          
          res <- sum_by_row2(res,columns = c(2:ncol(res)))
          res <- res[,c('gsim.no','sum')]
          names(res)[2] <- j
          res
        })
        res <- do.call(cbind,z1)
        res <- removedoubleColnames(res,'gsim.no')
      }
      
      # file.remove(files)
      write.table(res,
                  paste0(out_tiff_clean_flow,'/',land,'.Rres'), row.names = FALSE, sep = ",") ##write results to tile
    }
  }
}#close function


#a22.01.2021 -- check whether changes 
checkLAND <- function(land.list,
                      out_tiff_clean,
                      stat){
  
  for(flow in flow_lst){
    cat('...seaching for ',flow,'\n')
    out_tiff_clean_flow <- paste0(out_dir,'/Results_outTiffsRres/outTiffsRres_clean_',flow)

      files <- list.files(out_tiff_clean_flow,".Rres",full.names = TRUE)
      files <- files[grepl(paste(land.list, collapse = "|"), basename(files))]
      files <- files[!grepl("Natur|Second", basename(files))]
      # files <- files[as.numeric(lapply(land.list,function(x) grep(x,files)))]
      
      z1 <- lapply(c('pre_avg','post_avg'), function(j){
        print(j)
        z <- lapply(files,function(file){
          print(file)
          df <- read.table(file,header = TRUE,sep = ',')
          df <- df[,c('gsim.no',j)]
        })
        res <- as.data.frame(do.call(cbind,z))
        res <- removedoubleColnames(res,'gsim.no')

        res$sum <- as.numeric(lapply(1:nrow(res),function(i){
          val <- as.numeric(res[i, -which(names(res) %in% 'gsim.no')])
          val <- sum(val)
        }))
        res <- res[,c('gsim.no','sum')]
        names(res)[2] <- j
        res
      })
      res <- do.call(cbind,z1)
      res <- removedoubleColnames(res,'gsim.no')
      
      print(sapply(na.omit(res[,2:3]),stat))

  }
}#close function

























