

lib_lst <- c('purrr')  #'data.table','raster','foreach','doParallel','dplyr','purrr','rworldmap')

for (lib in lib_lst){
  if(!require(lib,character.only = TRUE)){install.packages(lib,dependencies = T); library(lib,character.only = TRUE)}}




#+++++++++++++++++++++++++ CALCULATE all the CHANGES +++++++++++++++++++++++++++++++++++++++++++

calc_changes <- function(Rres_file){
  
  VAR <- gsub(".Rres", "", basename(Rres_file))
  cat("... calculating for", VAR, "\n") 

  ### Get the pre- and post-changes from exacted variables
  df <- fread(Rres_file); sapply(df, class)

  # 16.01.2020 convert montly --> annual values for Precipitation/Evaporation/WaterVolume
  if(grepl("Precipitation|GPCC|EvapoTrans|WaterUse_|WaterUser|Irrigation", VAR)){
    df$pre_avg <- df$pre_avg*12
    df$post_avg <- df$post_avg*12
  }

  #-------------  calculate changes for each row ---------------------#

  for (i in 1:nrow(df)){
    gsim.no <- as.character(df$gsim.no[i]);   cat(VAR, i, "\n") 
    
  #1. call the values of change--------------------------------------#
   if(grepl("Dam", VAR)){
     if(grepl("Ratio", VAR)){ 
       post <- df$post_ratio[i]
       pre <- df$pre_ratio[i]
       } else {
         post <- df$cap_post[i]
         pre <- df$cap_pre[i]
     }
   } else {
      post <- df$post_avg[i]
      pre <- df$pre_avg[i]
    }
    
    
    #2. Different change between pre and post   ---------------------#
    diff <- post-pre
    
    #3. Absolute changes --------------------------------------------#
    absdiff <- abs(post-pre)
    
    #4. Percentage of changes ---------------------------------------#
    perc <- (post-pre)*100/pre

    #cal for climate
    if(VAR %in% c("Temperature")){
        perc <- ifelse(abs(pre) >= 100*abs(post) | abs(post) > 100*abs(pre),
                       100,
                       ifelse(pre<0&post<0,
                              (abs(post)-abs(pre))*100/abs(pre),
                              ifelse(pre<0 & post>0,
                                     (post-pre)*100/abs(pre),
                                     (post-pre)*100/pre)))
        }
    
    #calc for landcover/ratiodam (already in %)
    if(VAR %in% c("Grassland",
                  "Cropland", 
                  "Forest",
                  "Urbanland",
                  "NoVegetation") | grepl("Ratio", VAR)){
      perc <- post-pre
    }     
    
    #calc for wateruse, 
    if(VAR %in% c("WaterUse",
                  "Irrigation",
                  "Discharge",
                  "WaterVolume")) {
      perc <- ifelse(pre >= 100*post | post > 100*pre,
                     100,
                     (post-pre)*100/pre)
    }
    
    # ----------- put values to df --------------------------------#
    df$diff[i] <- diff
    df$absdiff[i] <- absdiff
    df$perc[i] <- perc
    
  }
return(df)
  
}#end of function


# 
# calc_changes <- function(Rres_file){
#   
#   VAR <- gsub(".Rres", "", basename(Rres_file))
#   cat("... calculating for", VAR, "\n") 
#   
#   ### Get the pre- and post-changes from exacted variables
#   df <- fread(Rres_file); sapply(df, class)
#   
#   # added 16.01.2020 convert montly --> annual values for Precipitation/Evaporation/WaterVolume
#   # if (VAR %in% c('Precipitation','EvapoTrans', wateruse)) {
#   if(grepl("Precipitation|GPCC|EvapoTrans|WaterUse_", VAR)){
#     df$pre_avg <- df$pre_avg*12
#     df$post_avg <- df$post_avg*12
#   }
#   #==========================  START CALCULATING ===================================#
#   
#   for (i in 1:nrow(df)){  #calculate for each row
#     gsim.no <- as.character(df$gsim.no[i]);   cat(VAR, i, "\n") 
#     
#     #1. call the values of change
#     if(grepl("Dam", VAR)){
#       if(grepl("Ratio", VAR)){ 
#         post <- df$post_ratio[i]
#         pre <- df$pre_ratio[i]
#       } else {
#         post <- df$cap_post[i]
#         pre <- df$cap_pre[i]
#       }
#     } else {
#       post <- df$post_avg[i]
#       pre <- df$pre_avg[i]
#     }
#     
#     
#     #2. Different change between pre and post   ----#
#     df$diff[i] <- post-pre
#     
#     #3. Absolute changes ----#
#     df$absdiff[i] <- abs(post-pre)
#     
#     #4. Percentage of changes ----#
#     #(controll  13.04.2020)
#     if(is.na(pre) == TRUE | is.na(post) == TRUE ){ #added 04.10 -  remove all value that are not 'real'
#       perc = NA
#       
#     } else {
#       #Landcover are in percentage --> no need to calpercentage 
#       # if(VAR %in% c(landcover, dam)) {
#       # if(grepl("land|rest.|egetation.|Ratio_", VAR)) { #replaced by below on 24.04.2023
#       if(VAR %in% c("Grassland",
#                     "Cropland", 
#                     "Forest",
#                     "Urbanland",
#                     "NoVegetation") | grepl("Ratio", VAR)){
#         perc <- post-pre
#         
#       }else {
#         #control if perc==0
#         if(pre == 0) {
#           perc <- post*100
#           
#         } else {
#           #calculate for water use/discharge/watervolume (where value might be very very small)
#           if(VAR %in% c("WaterUse",
#                         "Irrigation",
#                         "Discharge",
#                         "WaterVolume")) {
#             # if(grepl("WaterUse_|ischarg|olum", VAR)){ #replaced by above on 24.04.2023
#             # if(VAR %in% c(wateruse, waterlst)){
#             perc <- ifelse(pre >= 100*post | post > 100*pre,
#                            100,
#                            (post-pre)*100/pre)
#           }else {
#             if(grepl("emperatur|GPCC|EvapoTrans|", VAR)){
#               # if(VAR %in% climate){
#               # if(VAR =='Temperature'){ #in Temprature has some minus degree
#               if(grepl("emperatur", VAR)){
#                 perc <- ifelse(abs(pre) >= 100*abs(post) | abs(post) > 100*abs(pre),
#                                100,
#                                ifelse(pre<0&post<0,
#                                       (abs(post)-abs(pre))*100/abs(pre),
#                                       ifelse(pre<0 & post>0,
#                                              (post-pre)*100/abs(pre),
#                                              (post-pre)*100/pre)))
#               }else {
#                 perc <- (post-pre)*100/pre
#                 
#               }#close if-else for temperature
#             }#close for climate
#           }#close for wateruse
#         }#close for pre==0
#         
#         if(grepl("RandomVar",VAR)) perc <- (post-pre)*100/pre
#       }#close for landcover
#     } #close for NA pre post
#     df$perc[i] <- perc
#   }#finish whole files
#   
#   return(df)
#   
# }#end of function



#+++++++++++++++++++++++++++++++++++ COMBINE PRE-/POST ABSOLUTE VALUE +++++++++++++++++++++++++++++++
cbine_prepost <- function(VAR,
                          out_tiff_clean
                          ){
  # cat(VAR,'\n')
  Rres_file <- paste0(out_tiff_clean,'/',VAR,'.Rres'); VAR_DIR  
  
  ### Get the pre- and post-changes 
  df <- read.table(VAR_DIR,header = TRUE,sep=',')
  
  ifelse(VAR %in% dam,
         df <- df[colnames(df) %in% c('gsim.no','R_pre','R_post')],
         df <- df[colnames(df) %in% c('gsim.no','pre_avg','post_avg')])
  
  colnames(df)[2:3] <- c('Pre','Post')
  
  dt <- full_join(df,cont,'gsim.no')
  
  #----------------------------------- ADDED ON 16.01.2020 -----------------------------------------#
  #multiple 12 times for Precipitation/Evaporation/WaterVolume (they are monthly value)
  
  if (VAR %in% c('Precipitation','EvapoTrans',wateruse)) {
    dt$Pre <- dt$Pre*12
    dt$Post <- dt$Post*12
  }
  #---------------------------------------- DONE ---------------------------------------------------#

  #select the corresponding variable
  sel <- dt[colnames(dt) %in% c(p,'gsim.no','continent')]
  
  
  #----------------------------------- ADDED ON 09.01.2020 -----------------------------------------#
  #clean outlier for DAM --> avoid the significant impacts but no correlation 
  library(outliers)
  
  if (VAR %in% dam) {
    
    z <- list()
    for (c in cont_lst) {
      tb <- sel[sel$continent==c,]
      
      x <- tb[,2]
      
      if(length(x) != length(is.na(x))){
        #remove the outliers
        x1 <- na.omit(x); max(x1) #; plot(x1)
        
        #get the outliers
        outs <- scores(x1, type="chisq", prob=0.975) # beyond 90th %ile based on chi-sq
        mouts <- min(x1[outs]); mouts
        
        #outlier by quantile
        q3 <-  as.numeric(quantile(x1,probs = .975))
        #compare, if min remove smaller than q3 --> select
        if(mouts<q3){
          xout <- x1[outs]
          xoutmin <- min(xout)
        } else {
          xout <- x1[x1>q3]
          xoutmin <-q3
        }
        
        #replace the maximum value
        # x <- c(1, NA, 5, 9, 10, NA, 22,37); xout <- c(22, 37); xoutmin <- min(xout)
        for (i in 1:length(x)) {
          if (x[i] %in% xout ) {
            x[i] <- xoutmin }
        }; x; max(na.omit(x))
      }
      tb[,2] <- x
      z[[c]] <- tb
    }
    
    res <- do.call(rbind,z)
    row.names(res) <- res$gsim.no   
    rows <- as.character(sel$gsim.no)
    
    sel <- res[rows,]
  }
    sel <- sel[colnames(sel) %in% p]

  return(sel)
}#end of function















