

#setVAR: 
#set V0: original = full - static
#set V1: full = original + static
#set V2: full - (water use + dam)
#set V3: full - (water use + dam + static)

# dataFull: original dataframe having columns names

#1. select set of variables -------
selSet <- function(setVAR, 
                   dataFull){
  if(grepl("V0", setVAR)) VAR_rf <- dataFull[!grepl(paste(static, 
                                                          collapse = "|"), dataFull)]
  if(grepl("V1", setVAR)) VAR_rf <- dataFull
  if(grepl("V2", setVAR)) VAR_rf <- dataFull[!grepl(paste(c(dam, wateruse), 
                                                          collapse = "|"), dataFull)]
  if(grepl("V3", setVAR)) VAR_rf <- dataFull[!grepl(paste(c(dam, wateruse, static), 
                                                          collapse = "|"), dataFull)]
  return(VAR_rf)
}

#2. call data based on art of change ---------
callRres <- function(option,
                     out_dir,
                     rename_list = NULL){
  f <- paste0(out_dir,'/Res3_Main_Analysis/VarChanges_', option, '.Rres')
  if(!file.exists(f)) stop("find not found")
  if(grepl("pre|post", option)) f <- gsub("VarChanges_",
                                             "Values_",
                                             f)
  Rres <- as.data.frame(fread(f))
  
  if(!is.null(rename_list)){
    for(j in 1:ncol(Rres)){
      n <- names(Rres)[j]
      # if(!isTRUE(grepl("^[[:upper:]]", n))) {#+$ if want to test upper for both begin-end of string
      if(n %in% rename_list){
        n <- tools::toTitleCase(n)
      }
      names(Rres)[j] <- n
    }
  }
  return(Rres)
}




#3. clustering --------
getCluster <- function(data,
                       subCluster = NULL){
  
  data$clusters <- paste0(toupper(substr(data$cpt_clas,1,1)), data$dam_set)

  #remove stations having no KG zones
  data <- data[!is.na(data$KG_mainzone), ]
  
  if(!is.null(subCluster)){
    opt <- "D1"
    
    if(grepl("ong", subCluster)) opt <- "SD1"
    if(grepl("ing", subCluster)) opt <- "PD1"
    if(grepl("eak", subCluster)) opt <- "WD1"
      
    data$clusters <- ifelse(grepl(opt, data$clusters),
                            paste0(data$clusters, data$KG_zone), #if only 5 class, there should be _mainzone
                            data$clusters)
  }
  return(data)
}

getCluster_byCont <- function(data,
                              option,
                              optCPT = F){ #whole = 6 cont x 3 damset, strong = 6x3 only for strong CPT
  
  #automatically clustered by D1, D2, D3
  data$clusters <- paste0(toupper(substr(data$continent,1,2)), data$dam_set)
  data$clusters <- gsub("OD", "AD", data$clusters)
  
  if(isTRUE(optCPT)){
    data$clusters <- paste0(toupper(substr(data$continent,1,2)),
                            toupper(substr(data$cpt_clas,1,1)))
    data$clusters <- gsub("NO", "NA", data$clusters)
    data$clusters <- gsub("SO", "SA", data$clusters)
  }
  
  #select only strongcpt
  if(option == "strong"){
    data <- data[data$cpt_clas == "strong", ]
  }
  
  return(data)
}



# 4. RandomForest ----------
calcRF <- function(data,
                     targetVar, #water
                     option,
                     m = 8, #da test tu 10-13; pattern khong thay doi nhieu! --  should be 5-25
                     trainOpt, #NULL=train, "n
                   ptrain
                  ){
  
  # names(data)[which(names(data) %in% option)] <- "clusters"
  clus_lst <- as.data.frame(table(data$clusters))
  clus_lst <- as.character(clus_lst$Var1); clus <- clus_lst[1]
  
  z <- list(); k=list(); kn <- list()
  for(clus in clus_lst){
    #subset
    print(clus)
    subDT <- data[data$clusters == clus, VAR_rf]
    
    #---------------------- 1. Train RF  --------------------------------------#
    library(randomForest)
    library(caret)
    
    colnames(subDT)[colnames(subDT) == targetVar] <- 'water'
    
    if(nrow(subDT) <= 10){
      res_varImpt <- data.frame(VAR = VAR_rf[!grepl(water, VAR_rf)],
                                importance = NA)
      
      k[[clus]] <- NA
      kn[[clus]] <- NA
    } else {
      
      ## Split the data so that we use 70% of it for training
      train_index <- createDataPartition(y = subDT$water, p = ptrain, list = FALSE) #y=subRres$continent
      
      ## Subset the data
      trainDT <- na.omit(subDT[train_index, ])
      if(!is.null(trainOpt)) trainDT <- na.omit(subDT)
      
      #test data
      testDT <- na.omit(subDT[-train_index, ])
      
      rf <- randomForest(water~., data = trainDT, ntree = 500, mtry = m, importance = TRUE,na.action = na.omit) # na.action = na.roughfix
      #Export Importance
      res_varImpt <- as.data.frame(importance(rf))
      res_varImpt$VAR <- row.names(res_varImpt)
      res_varImpt$importance <- round(res_varImpt$IncNodePurity*100/sum(res_varImpt$IncNodePurity),1)
      res_varImpt <- res_varImpt[, c('VAR','importance')]
      
      #----------- 2. Predict data and KGE values -------------------------------#
      ## Generate predictions
      pred <- predict(object = rf, 
                      newdata = testDT[, -which(grepl("water", names(testDT)))])
      
      library(hydroGOF)
      k[[clus]] <- hydroGOF::KGE(sim = pred,
                                 obs = testDT$water)
      kn[[clus]] <- hydroGOF::NSE(sim = pred,
                                  obs = testDT$water)
    }
    
    
    #------------------------------------------------------------------------------------#
    z[[clus]] <- res_varImpt
  }
  
  res <- list(k, kn, z)
  return(res)
}

#5.  save to results ----
#k, kn, z

saveRFres <- function(res_RF,
                      option = "RF"){
  
  #results of RF
  if(option == "RF"){
    res <- as.data.frame(do.call(cbind,
                                 as.data.frame(res_RF[3])))
    res <- res[,-seq(3,ncol(res)-1,2)]
    names(res) <- gsub(".VAR|.importance", "", names(res))
    names(res)[1] <- "VAR"
    }

  #--- KGE -----#
  if(option == "KGE"){
    res <- as.data.frame(do.call(rbind,
                                 as.data.frame(res_RF[1])))
    colnames(res) <- "KGE"
    res$Clusters <- rownames(res)
  }
  #--- NSE -----#
  if(option == "NSE"){
    res <- as.data.frame(do.call(rbind,
                                 as.data.frame(res_RF[2])))
    colnames(res) <- "NSE"
    res$Clusters <- rownames(res)
    
  }
  return(res)
}






#6. rename clusters ----
#used in GA3_Q2
renameClus <- function(x){
  if(grepl("SD1", x)){
    xnew <- switch (x,
                    SD1A =  "S-D1-Equa",
                    SD1B =  "S-D1-Arid",
                    SD1Cf = "S-D1-WmFh",
                    SD1Cs = "S-D1-WmSd",
                    SD1Cw = "S-D1-WmWd",
                    SD1Df = "S-D1-SnFh",
                    SD1Ds = "S-D1-SnSd",
                    SD1Dw = "S-D1-SnWd",
                    SD1E ="S-D1-Polar"
    )
  } else {
    x= strsplit(x, "D")[[1]]
    xnew = paste0(x[1], "-D", x[2])
  }
  return(xnew)
}




#7. Get levels for clusters ------
#used in GA3_Q2
getLevelsClusters <- function(clusList){
  newlist <- factor(clusList,
                    level = c("S-D1-Equa",
                              "S-D1-Arid",
                              "S-D1-WmFh",  
                              "S-D1-WmSd",
                              "S-D1-WmWd",
                              "S-D1-SnFh",
                              "S-D1-SnSd",
                              "S-D1-SnWd",
                              "S-D1-Polar",
                              "S-D2",
                              "S-D3",
                              "P-D1",
                              "P-D2",
                              "P-D3" ,
                              "W-D1",
                              "W-D2",
                              "W-D3" 
                    ),
                    ordered = T)
  return(newlist)
}















































































