
# ==========================================================================#
#Replace a vector of string by another one
gsub2 <- function(pattern, replacement, x, ...) {
  for(i in 1:length(pattern))
    x <- gsub(pattern[i], replacement[i], x, ...)
  x
}


# ========================== A. BASEWORLD for global plot ==================================
# 1. global normal theme ----
baseworld <- function(){
  library(ggplot2)
  world_map <- map_data("world")
  base_world <-ggplot() + 
    coord_fixed(ylim = c(-55,80),xlim = c(-170,175)) +  xlab("") + ylab("") + 
    geom_polygon(data=world_map, aes(x=long, y=lat, group=group), 
                 colour="grey50", fill="grey90") + 
    scale_x_continuous(limits = c(-175,175), breaks = seq(-175,175,25))   + 
    scale_y_continuous(limits = c(-60,80), breaks = seq(-60,80,20))   + 
    theme(panel.background = element_rect(fill = 'white', colour = 'black'), 
          axis.line = element_blank(), 
          plot.title = element_blank(),
          axis.text = element_text(size = 40) , 
          panel.grid.major = element_line(color = "grey"),
          axis.ticks = element_line(color = 'grey')); base_world          # to hide vach ke in x/y-axis
}


# 2. global dark theme ----
baseworld2 <- function(){
  library(ggplot2)
  world_map <- map_data("world")
  base_world <-ggplot() + 
    coord_fixed(ylim = c(-55,80),xlim = c(-170,175)) +  xlab("") + ylab("") + 
    geom_polygon(data=world_map, aes(x=long, y=lat, group=group), 
                 colour="dimgrey", fill="dimgrey") + 
    scale_x_continuous(limits = c(-175,175), breaks = seq(-175,175,50))   + 
    scale_y_continuous(limits = c(-60,80), breaks = seq(-60,80,40))   + 
    theme(panel.background = element_rect(fill = 'grey21', colour = 'black'), 
          axis.line = element_blank(), 
          plot.title = element_blank(),
          axis.text = element_text(size = 40) , 
          panel.grid.major = element_line(color = "grey", linetype = 2),
          panel.grid.minor = element_blank(),
          axis.ticks = element_line(color = 'grey')); base_world          # to hide vach ke in x/y-axis
}


# ========================== >>> B <<< . THEME for global plot ==================================
themeCPT_global <- function(pts_size,
                            color_lst,
                            title){
  list(
    ylab("lat"),
    xlab("long"),
    scale_size_discrete(range = pts_size, guide=NULL),
    scale_colour_manual(values = color_lst),
    
    guides(color=guide_legend(override.aes = list(size = 10),        # edit size of symbol in legend
                              title = title,
                              direction = "vertical", title.position = "top",
                              label.hjust = 0, label.vjust = 0.5, 
                              ncol = 1)),
    theme(legend.position= c(0.15,0.25),
          legend.text=element_text(size=40),
          legend.title=element_text(size=45,face = 'bold'),
          legend.key.height=unit(2,"cm"),
          legend.key.size = unit(2,'cm'),
          axis.title=element_blank())
  )
}





RF_theme <- function(high.color){
  list(
    scale_fill_gradient2(low = "darkorange2", mid = "white", high = high.color,na.value = "grey85" ,
                         guide = "colourbar", aesthetics = "fill",
                         limits = c(0,100),
                         breaks=seq(0,100,10)),
      
      scale_y_discrete() ,
      ylab("") , ggtitle("") ,
      #                          )) ,
      theme_bw() , 
    theme(legend.position= "bottom",
                          legend.text = element_text(size = 15),
                          legend.title = element_text(size = 15,face = 'bold'),
                          plot.title = element_text(size = 15,face = 'bold'),
                          axis.title.x =element_blank(),
                          axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                          axis.text.y = element_text(size = 12),
                          axis.text.x = element_text(size = 12,angle = 0,hjust = 0.5),
                          panel.grid.major.x =  element_blank(),
                          panel.grid.minor = element_blank(),
                          strip.background = element_blank(),strip.text = element_text(size = 13,face = "bold"))
  )
}

RF_theme_differenceRandomVar <- function(high.color,
                                         ymin,ymax,breaks,
                                         legend.tit,
                                         legend.pos){
  list(
    scale_fill_gradient2(low = "darkorange2", mid = "white", high =high.color,na.value = "grey85" ,
                         guide = "colourbar", aesthetics = "fill",
                         limits = c(ymin,ymax),
                         breaks=seq(ymin,ymax),breaks),
    scale_x_discrete(labels = function(x) str_wrap(x, width = 10)) ,
    ylab("") ,
    guides(fill = guide_colourbar(label.theme = element_text(colour = 'darkred', angle = 0,size = 15),
                                  title = legend.tit,direction = "horizontal", title.position = "left",
                                  title.hjust = 0.5,
                                  label.hjust = 0.5, label.vjust = 1)) ,
    theme_bw() ,  theme(legend.position= legend.pos,
                        legend.text = element_text(size = 15),
                        legend.key.width = unit(1.5,'cm'),
                        legend.title = element_text(size = 15,face = 'bold'),
                        plot.title = element_text(size = 15,face = 'bold'),
                        axis.title.x =element_blank(),
                        axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                        axis.text.y = element_text(size = 15),
                        axis.text.x = element_text(size = 15,angle = 90,hjust = 1),
                        panel.grid.major.x =  element_line(colour='grey'),
                        panel.grid.minor = element_blank(),
                        strip.background = element_blank(),
                        strip.text = element_text(size = 15,face = "bold"))
  )
}
  
boxplot_theme <- function(legend.tit,
                          legend.pos){
  list(
    guides(fill=guide_legend(override.aes=list(size = 0.2),        # edit size of symbol in legend
                              title = legend.tit,
                              direction = "vertical", title.position = "left",
                              label.hjust = 0, label.vjust = 0.5,
                              nrow = 1)),
      theme(legend.position = legend.pos,    # c(0.2,0.91)
            legend.key.width = unit(1.5,"line"),
            legend.title = element_text(size = 18,face = "bold"),
            legend.text = element_text(size = 18),
            plot.title = element_text(size = 22,face = "bold"),  #,margin = margin(t = 15, b = -25),hjust = 0.05
            plot.margin = unit(c(1,0.5,0,0.5), "cm"),   #refer to t-r-b-l
            axis.title.y =element_text(size = 18,face = 'bold'),
            axis.title.x = element_blank(),
            axis.text = element_text(size = 16), 
            strip.background = element_rect(color = 'grey'),strip.text = element_text(size = 15,face = "bold"),
            panel.border = element_rect(fill = NA),
            panel.grid.major.y = element_line(color = "grey") ,
            panel.background = element_rect(fill = NA))
  )
}


dotstoWorldmap <- function(data,
                                x,
                                y,
                                title,
                                dot.size,
                                colour.by,
                                colour.brewer,
                                legend.tit,
                                legend.pos
                                ){
  names(data)[which(names(data) == x)] <- "x"
  names(data)[which(names(data) == y)] <- "y"
  names(data)[which(names(data) == colour.by)] <- "colour.by"

  p <-
    baseworld() +
      geom_point(data = data,
                 aes(x,y,
                     colour= factor(colour.by)),
                 size =  dot.size)+
    theme(legend.position= legend.pos,
          legend.text=element_text(size=30),
          legend.title=element_text(size=35,face = 'bold'),
          plot.title = element_text(size = 35,face = 'bold',margin = margin(b = -45,l=-40)),
          legend.key.height=unit(2,"cm"),
          legend.key.size = unit(2,'cm'),
          axis.text = element_blank(),
          axis.title.y = element_text(size = 45)); p
  
  if(grepl('bottom|top',legend.pos)){
    p <- p +  guides(color=guide_legend(override.aes=list(size = 10),        # edit size of symbol in legend
                                      title = paste0(legend.tit,":"),
                                      direction = "horizontal", title.position = "left",
                                      label.hjust = 0, label.vjust = 0.5, 
                                      nrow = 1))
    } else {
      p <- p + guides(color=guide_legend(override.aes=list(size = 10),        # edit size of symbol in legend
                                title = paste0(legend.tit,":"),
                                direction = "horizontal", title.position = "top",
                                label.hjust = 0, label.vjust = 0.5,
                                ncol = 1))

    }
  p <- p + scale_colour_brewer(palette = colour.brewer)
  return(p)
}
  

clusterstoWorldmap <- function(data,
                           x,
                           y,
                           title,
                           dot.size,
                           colour.by,
                           colour.brewer,
                           legend.tit,
                           legend.pos,
                           map.tit){
  names(data)[which(names(data) == x)] <- "x"
  names(data)[which(names(data) == y)] <- "y"
  names(data)[which(names(data) == colour.by)] <- "colour.by"
  
  p <-
    baseworld() +
    geom_point(data = data,
               aes(x,y,
                   colour= factor(colour.by)),
               size =  dot.size)+
    theme(legend.position= legend.pos,
          legend.text=element_text(size=30),
          legend.title=element_text(size=35,face = 'bold'),
          plot.title = element_text(size = 35,face = 'bold',margin = margin(b = -45,l=-40)),
          legend.key.height=unit(2,"cm"),
          legend.key.size = unit(2,'cm'),
          axis.text = element_blank(),
          axis.title.y = element_text(size = 45)) +
    ggtitle(map.tit); p

    p <- p + guides(color=guide_legend(override.aes=list(size = 10),        # edit size of symbol in legend
                                       title = paste0(legend.tit,":"),
                                       direction = "horizontal", title.position = "top",
                                       label.hjust = 0, label.vjust = 0.5,
                                       ncol = 1))
  
  p <- p + scale_colour_brewer(palette = colour.brewer)
  return(p)
}



#plot heatmap for RF - whole group -----
plotRFheatmap <- function(data,
                          x,
                          y,
                          fill,
                          label,
                          high.colour = "cyan4",
                          text.colour = "black",
                          facet,
                          ymin,ymax,
                          breaks,
                          legend.tit,
                          legend.pos
){
  #set name
  names(data)[which(names(data) == x)] <- "x"
  names(data)[which(names(data) == y)] <- "y"
  names(data)[which(names(data) == fill)] <- "fill"
  if(label != fill){
    names(data)[which(names(data) == label)] <- "label"
  }else{
    data$label <- data$fill
  }    
  
  if(!is.null(facet)) names(data)[which(names(data) == facet)] <- "facet"
  
  
  ymin <- ifelse(ymin > -90, round(ymin, -1), round(ymin, -2))
  ymax <- ifelse(ymax < 90, round(ymax, -1), round(ymax, -2))

  p <-
    ggplot(data, aes(x,y)) +
    geom_tile(aes(fill = fill))  +
    geom_text(aes(label=label),color=text.colour,size=4.5)+
    
    scale_fill_gradient2(low = "darkorange2", mid = "white", high =high.colour, na.value = "grey85" ,
                         guide = "colourbar", aesthetics = "fill",
                         limits = c(ymin,ymax),
                         breaks=seq(ymin,ymax,breaks)
    ) +
    
    scale_x_discrete(labels = function(x) str_wrap(x, width = 10)) +
    ylab("") +
    guides(fill = guide_colourbar(label.theme = element_text(colour = 'darkred', angle = 0,size = 15),
                                  title = legend.tit,direction = "horizontal", title.position = "left",
                                  title.hjust = 0.5,
                                  label.hjust = 0.5, label.vjust = 1)) +
    theme_bw() +  theme(legend.position= legend.pos,
                        legend.text = element_text(size = 15),
                        legend.key.width = unit(1.5,'cm'),
                        legend.title = element_text(size = 15,face = 'bold'),
                        plot.title = element_text(size = 15,face = 'bold'),
                        axis.title.x =element_blank(),
                        axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                        axis.text.y = element_text(size = 15),
                        axis.text.x = element_text(size = 15,angle = 0,hjust = 0.5),
                        panel.grid.major.x =  element_line(colour='grey'),
                        panel.grid.minor = element_blank(),
                        strip.background = element_blank(),
                        strip.text = element_text(size = 15,face = "bold")) ; p
  if(!is.null(facet)){
    p <- p + facet_grid(.~facet, scales="free", space="free"); p
  }
  
  return(p)
}

# Theme for Fig.7 (paper 3) ----
myTheme <- function(){
  list(
    scale_x_discrete(labels = function(x) str_wrap(x, width = 10)),
    xlab(""),
    guides(fill=guide_legend(override.aes=list(size = 0.2),        # edit size of symbol in legend
                             title = "",
                             direction = "vertical", title.position = "left",
                             label.hjust = 0, label.vjust = 0.5, 
                             nrow = 1)),
    theme(legend.position = "bottom",
          #legend.key.height = unit(2,"line"),
          legend.title = element_text(size = 18,face = "bold"),
          legend.text = element_text(size = 18),
          plot.title = element_text(size = 22,face = "bold",margin = ggplot2::margin(t = 15, b = 5),hjust = 0.05),
          plot.margin = unit(c(1,0.5,0,0.5), "cm"),   #refer to t-r-b-l
          axis.title.y =element_text(size = 18,face = 'bold'),
          axis.title.x = element_blank(),
          axis.text.y= element_text(size = 16),             # to hide text in x-axis
          axis.text.x = element_text(size = 13),
          #axis.ticks.x=element_blank(),
          panel.border = element_rect(fill = NA),
          panel.grid.major = element_line(color = "grey"),
          panel.background = element_rect(fill = NA))
  )
}


# Theme for Fig.8 - combine ----
myTheme2 <- function(){
  list(
    scale_x_discrete(labels = function(x) str_wrap(x, width = 10)),
    xlab(""),
    # guides(shape = guide_legend(override.aes=list(size=6,shape=shape_lst,fill = fill_lst),
    #                             keywidth=0.3,keyheight=0.5, default.unit="inch",  #keyheight: chieu cao cua text va symbol trong plot, keywidth: chieu rong cua text
    #                             title = "Changes: ",
    #                             direction = "vertical", title.position = "left",title.vjust = 1,
    #                             label.hjust = 0, label.vjust = 0.5, 
    #                             nrow = length(dir_lst))) +
    theme(legend.position = "none",
          #legend.key.height = unit(2,"line"),
          legend.title = element_text(size = 18,face = "bold"),
          legend.text = element_text(size = 18),
          plot.title = element_text(size = 22,face = "bold",margin = ggplot2::margin(t = 15, b = 5),hjust = 0.05),
          plot.margin = unit(c(1,0.5,0,0.5), "cm"),   #refer to t-r-b-l
          axis.title.y =element_text(size = 18,face = 'bold'),
          axis.title.x = element_blank(),
          axis.text.y= element_text(size = 16),             # to hide text in x-axis
          axis.text.x = element_blank(),
          #axis.ticks.x=element_blank(),
          panel.border = element_rect(fill = NA),
          panel.grid.major = element_line(color = "grey"),
          panel.background = element_rect(fill = NA))
  )
}

# Theme for Fig.7 (paper 3) ----
dam_theme <- function(){
  list(
    geom_violin(aes(fill=continent)),
    geom_boxplot(width=0.1) , #geom_density(jitter_point=TRUE),
      scale_fill_brewer(palette = "RdBu") , 
      ylab("") ,
      xlab("") , theme_bw(),
      
      theme(legend.position ="none",
            plot.title = element_text(size = 20),    # gia tri cu 14
            axis.title.y =element_text(size = 13, face = "bold", angle = 90, vjust = 0.3,hjust = 0.45),    # to show sub-title in y-axis
            axis.title.x = element_text(size = 13,face = 'bold',hjust = 0.5),            # to hide sub-title in x-axis
            axis.text.y = element_text(size = 13),
            axis.text.x= element_text(size = 13),             # to hide text in x-axis
            axis.ticks.x=element_blank()) ,           # to hide vach ke in x-axis
      labs(fill = "")
  )
}



shape_info <- function(dir_lst){
  if(length(dir_lst) ==3){
    shape_lst <- c(25,4,24)
  } else {
    if (length(dir_lst) ==2) {
      if((dir_lst[1]==-1 & dir_lst[2]== 1)) {
        shape_lst <- c(25,24)
      } else {
        if((dir_lst[1]==-1 & dir_lst[2]== 0)) {
          shape_lst <- c(25,4)
        } else {
          shape_lst <- c(4,24)
        }
      }
    } else {
      if(dir_lst=="1") {
        shape_lst <- c(25)
      } else {
        if(dir_lst=="-1") {
          shape_lst <- c(24)
        } else {
          shape_lst <- c(4)
        }
      }
    }
  }
  
  shape_lst
}


label_info <- function(dir_lst){
  if(length(dir_lst) ==3){
    label_lst <-  c('Significant decrease','Not significant',"Significant increase")
  }   else {
    if (length(dir_lst) ==2) {
      if((dir_lst[1]==-1 & dir_lst[2]== 1)) {
        label_lst <-  c('Significant decrease',"Significant increase")
      } else {
        if((dir_lst[1]==-1 & dir_lst[2]== 0)) {
          label_lst <-  c('Significant decrease','Not significant') 
        } else {
          label_lst <-  c('Not significant',"Significant increase") 
        }
      }
    } else {
      if(dir_lst=="1") {
        label_lst <-  c("Significant increase") 
      } else {
        if(dir_lst=="-1") {
          label_lst <-  c('Significant decrease') 
        } else {
          label_lst <-  c('Not significant') 
        }
      }
    }
  }
  label_lst
}


fill_info <- function(dir_lst){
  if(length(dir_lst) ==3) {
    fill_lst <- c('red',"grey90",'green')
  } else {
    if (length(dir_lst) ==2) {
      if((dir_lst[1]==-1 & dir_lst[2]== 1)) {
        fill_lst <- c('red','green')
      } else {
        if((dir_lst[1]==-1 & dir_lst[2]== 0)) {
          fill_lst <- c('red','grey90')
        } else {
          fill_lst <- c('grey90','green')
        }
      }
    } else {
      if(dir_lst=="1") {
        fill_lst <- c('green')
      } else {
        if(dir_lst=="-1") {
          fill_lst <- c('red')
        } else {
          fill_lst <- c('grey90')
        }
      }
    }
  }
}


fig7_theme <- function(){
  list(
    guides(shape = guide_legend(override.aes=list(size=6,shape=shape_lst,fill = fill_lst),
                                keywidth=0.3,keyheight=0.5, default.unit="inch",  #keyheight: chieu cao cua text va symbol trong plot, keywidth: chieu rong cua text
                                title = "Changes: ",
                                direction = "vertical", title.position = "left",title.vjust = 1,
                                label.hjust = 0, label.vjust = 0.5, 
                                nrow = length(dir_lst)),
           color=guide_legend(override.aes=list(size=1),        # edit size of symbol in legend
                              keywidth=0.3,keyheight=0.5, default.unit="inch",  #change space between lines in legend
                              title = 'Period: ',
                              direction = "vertical", title.position = "left",
                              label.hjust = 0, label.vjust = 0.5, title.vjust = 1,
                              nrow = 2)),
    theme_bw(),
    theme(legend.position= "none",
                          legend.text = element_text(size = 25),
                          legend.title = element_text(size = 25,face = 'bold'),
                          plot.title = element_text(size = 25,face = 'bold',hjust = 0.5),
                          axis.title.x =element_blank(),
                          axis.title.y =element_text(size = 30, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                          axis.text.y = element_text(size = 25),
                          axis.text.x = element_text(size = 25,angle = 90,hjust = 1),
                          panel.grid.major.x =  element_blank(),
                          panel.grid.minor.x = element_blank(),
                          strip.background = element_rect(fill = "grey80"),strip.text = element_text(size = 30))
  )
}

fig7_heatmap_theme <- function(){
  list(
    guides(fill=guide_legend(override.aes=list(size=2),        # edit size of symbol in legend
                             keywidth=0.3,keyheight=0.2, default.unit="inch",  #change space between lines in legend
                             title = 'Changes: ',
                             direction = "vertical", title.position = "left",
                             label.hjust = 0, label.vjust = 0.5, 
                             nrow = 1)),
      theme_bw(),  theme(legend.position= "bottom",
                          legend.text = element_text(size = 15),
                          legend.title = element_text(size = 15,face = 'bold'),
                          plot.title = element_text(size = 15,face = 'bold'),
                          axis.title.x =element_blank(),
                          axis.title.y =element_text(size = 15, face = "bold", angle = 90, vjust = 0.5),    # to show sub-title in y-axis
                          axis.text.y = element_text(size = 12),
                          axis.text.x = element_text(size = 12,angle = 0,hjust = 0.5),
                          panel.grid.major.x =  element_blank(),
                          panel.grid.minor = element_blank(),
                          strip.background = element_blank(),strip.text = element_text(size = 13,face = "bold"))
  )
}

