


changeGroupName <- function(list){
  new.list <- as.character(
    lapply(list,function(x) switch(x,
                                   SND = 'Strong-NoDam',
                                   SWD = 'Strong-WithDam',
                                   PND = 'Pending-NoDam',
                                   PWD = 'Pending-WithDam',
                                   WND = 'Weak-NoDam',
                                   WWD = 'Weak-WithDam')
    ))
  
}

changeGroupName_back <- function(list){
  new.list <- as.character(
    lapply(list,function(x) switch(x,
                                   'Strong-NoDam' = "SND",
                                   'Strong-WithDam'= "SWD",
                                   'Pending-NoDam'= "PND",
                                   'Pending-WithDam'= "PWD",
                                   'Weak-NoDam'= "WND",
                                   'Weak-WithDam'= "WWD")
    ))
  
}




setCategory <- function(data){
  category <- ifelse(data %in% climate,
                     "Climate",
                     ifelse(data %in% landcover,
                            "Landcover",
                            ifelse(data %in% 'RandomVar',
                                   "RandomVar",
                                   "Water Infrastructure")))
  category
}



removeClusters <- function(data,
                           min.basin = 12){
  clus.col <- grep("clus|clusters|clust",names(data))
  clus.name <- names(data)[clus.col]
  
  names(data)[clus.col] <- "clust"
  clus <- as.data.frame(table(data$clust))
  clus <- clus[clus$Freq > min.basin,]
  clus$new <- 1:nrow(clus)
  
  data$clust <- as.numeric(
    lapply(data$clust,function(x){
      if(x %in% clus$Var1){
        xnew <- clus$new[clus$Var1 == x]
      } else {
        xnew <- NA
      }
      xnew
    }))
  # data <- data[!is.na(data$clust),]
  names(data)[clus.col] <- clus.name
  return(data)
}





runRandomForest <- function(data,
                            var,
                            ntree,
                            mtry,
                            size.min){
  #prepare empty function
  dtt <- data.frame(VAR = c(climate,landcover,wateruse,dam,'RandomVar'))
  
  #rename the column --> set the same name for discharge/watervolume
  colnames(data)[colnames(data)==var] <- 'water'
  if(nrow(data) < size.min){
    dt <- data.frame(VAR = dtt$VAR,
                     var_importance= NA)
  }else{
    m <- randomForest(water~., data = data, ntree = ntree, mtry = mtry, importance = TRUE,na.action = na.omit)
    
    #Export Importance
    dt <- as.data.frame(importance(m))
    dt$VAR <- row.names(dt)
    dt$var_importance <- round(dt$IncNodePurity*100/sum(dt$IncNodePurity),1)
    
    dt <- dt[colnames(dt) %in% c('VAR','var_importance')]#; dt
    dt <- dplyr::left_join(dtt,dt,'VAR')
  }
  return(dt)
}


selectDatabyDam <- function(data,
                            class,
                            randomvar = TRUE){
  if(grepl('no|No',class)){
   select <- c(climate,landcover,wateruse,water,'RandomVar')
  } else {
   select <- c(climate,landcover,wateruse,ratiodam,water,'RandomVar')
  }
  if(isTRUE(randomvar) == FALSE) select <- select[!grepl('RandomVar',select)]
  data <- data[,select]
  return(data)
}



diffRandomVar <- function(data,
                          var,
                          column){
  nr <- grep(var,data[,column])
  for(j in 1:ncol(data)){
    if(j == column){
      data[,j] <- data[,j]
    } else {
      val <-  data[nr,j]
      for(i in 1:nrow(data)){
        data[i,j] <- data[i,j] - val
      }
    }
  }
  return(data)
}

