

#no  more needed functions
#this is used to extract capacity of dam in each basin, directly from shapefile, based on change point
#this is now changed by precook dam
extrCapDam <- function(flow,
                       dataSet,
                       damSet,
                       CAP,
                       cptPath,
                       Qpath,
                       RDATADIR,
                       grand_shp,
                       out_dir){
  out_dir_var <- paste0(out_dir,'/Res2_outTiffsRres/outTiffsRres_set', dataSet,"_",flow) 
  if(!file.exists(out_dir_var)) dir.create(out_dir_var)
  
  #get the change point detection
  gsim_cpt <- read.table(paste0(cptPath,"/Final_CPT_set", dataSet,"_",flow,".Rres"), header = TRUE, sep = ",")
  gsim_lst <- as.character(gsim_cpt$gsim.no)
  
  noRdata_files <- readLines(paste0(path,"/noRdata.lst"))
  
  
  results <- list()
  # results <- foreach(i=gsim_lst, .errorhandling = "stop", .packages = c("raster", "data.table"), .verbose=F) %dopar% {
  for (i in gsim_lst){
    ### Print the current ID to follow the process
    cat("running",paste0(flow,' - basin ',match(i,gsim_lst)," (",i,")"), "\n")
    
    ### Get the change point year (cpt_ID) of each station
    cpt_ID <- gsim_cpt$cpt_ID[match(i,gsim_cpt$gsim.no)]  
    # yearstart <- gsimFull[match(i,gsimFull$gsim.no),'year.start'] 
    
    ### (ERROR_2): NO SHAPEFILE ---> can't find the dam on it
    if (i %in% noRdata_files) {  #----------------------------- No Rdata files/shp available, return to NA resuls
      print('NA - Rdata files not available')
      res <- data.frame(gsim.no = i,
                        cpt_ID,
                        dam.num=NA,
                        
                        dam_w_pre = NA,
                        dam_w_post = NA,
                        dam_nw_pre = NA,
                        dam_nw_post = NA,
                        
                        dam_w_sum = NA,
                        dam_nw_sum = NA,
                        
                        cap_w_pre = NA,
                        cap_w_post= NA,
                        cap_nw_pre = NA,
                        cap_nw_post = NA,
                        
                        cap_w_sum = NA,
                        cap_nw_sum = NA,
                        first_dam = NA, last_dam = NA)
      res$note <- 'NA_shps'  #'Non-sig changepoint'
      
    } else {  
      ### Get the basin shape (pre-cooked as Rdata file to speed up)
      load(paste0(RDATADIR, "/",  gsub(".shp", "",i), ".RData"))
      # plot(tmp_shp)
      
      #clip the point which falls into the polygon
      # c <- as.data.frame(point.in.poly(grand_shp,tmp_shp)) 
      
      #find all dams located inside basin
      dam <- as.data.frame(
        sp::over(tmp_shp, grand_shp, returnList = TRUE)
      )
      colnames(dam) <- gsub("X0.","", colnames(dam))
      
      #select few colums
      dam <- dam[colnames(dam) %in% c('GRAND_ID','YEAR','MAIN_USE',CAP)]
      dam.num <- length(dam$GRAND_ID) #number of dam
      
      #-------------- SELECT DAMS in studied period -----------------------------------#
      if(grepl('B', damSet)) dam <- dam[dam$YEAR >= 1960 & dam$YEAR <= 2010,]
      if(grepl('C', damSet)) dam <- dam[dam$YEAR < 1960,]
      if(grepl('D', damSet)) dam <- dam[dam$YEAR < 2010,]
      
      #--------------------------------------------------------------------------------#
      x <- dam$YEAR
      
      if(dam.num==0) {
        res <- data.frame(gsim.no = i,
                          cpt_ID,
                          dam.num=0,
                          
                          dam_w_pre = NA,
                          dam_w_post = NA,
                          dam_nw_pre = NA,
                          dam_nw_post = NA,
                          
                          dam_w_sum = NA,
                          dam_nw_sum = NA,
                          
                          cap_w_pre = NA,
                          cap_w_post= NA,
                          cap_nw_pre = NA,
                          cap_nw_post = NA,
                          
                          cap_w_sum = NA,
                          cap_nw_sum = NA,
                          first_dam = NA, last_dam = NA)
        res$note <-  'nodam'
        
      } else {
        #find the dam with using withdraw water 
        wd <- dam[dam$MAIN_USE %in% c('Irrigation','Water supply'),]
        
        ### lsit the dam that return water to the rivers
        nowd <- anti_join(dam, wd,by='GRAND_ID')
        
        ### ----------------- added on 27.01.2020 --------------- to find whether the dam is fully before the discharge series
        ### (ERROR 3): NO CHANGE POINTS but still list dam
        if(is.na(cpt_ID)) {
          res <- data.frame(gsim.no = i,
                            cpt_ID,
                            dam.num,
                            
                            dam_w_pre = NA,
                            dam_w_post = NA,
                            dam_nw_pre = NA,
                            dam_nw_post = NA,
                            
                            dam_w_sum = nrow(wd),
                            dam_nw_sum = nrow(nowd),
                            
                            cap_w_pre = NA,
                            cap_w_post= NA, 
                            cap_nw_pre = NA,
                            cap_nw_post = NA,
                            
                            cap_w_sum = sum(wd$CAP_MCM),
                            cap_nw_sum = sum(nowd$CAP_MCM),
                            first_dam=min(x),last_dam=max(x)) 
          #--------------------------------------------------------------------------------#
        } else{
          res <- data.frame(gsim.no = i,
                            cpt_ID,
                            dam.num,
                            
                            dam_w_pre = nrow(wd[wd$YEAR<cpt_ID,]),
                            dam_w_post = nrow(wd[wd$YEAR>=cpt_ID,]),
                            dam_nw_pre = nrow(nowd[nowd$YEAR<cpt_ID,]),
                            dam_nw_post = nrow(nowd[nowd$YEAR>=cpt_ID,]),
                            
                            dam_w_sum = nrow(wd),
                            dam_nw_sum = nrow(nowd),
                            
                            cap_w_pre = sum(wd$CAP_MCM[wd$YEAR<cpt_ID]),
                            cap_w_post= sum(wd$CAP_MCM),
                            cap_nw_pre = sum(nowd$CAP_MCM[nowd$YEAR<cpt_ID]),
                            cap_nw_post = sum(nowd$CAP_MCM),
                            
                            cap_w_sum = sum(wd$CAP_MCM),
                            cap_nw_sum = sum(nowd$CAP_MCM),
                            first_dam=min(x),last_dam=max(x)) 
        }#close if-else for non-cptID
        res$note <- '-'
        
      }#close if-else for no-dam
    }#close ifelse for no Rdata
    results[[i]] <- res
  }# close foreach
  all_res <- do.call(rbind,results)#rbindlist(results) 
}#end of function
